#!/usr/bin/env python3
"""
Система тестирования ботов

ИСПОЛЬЗОВАНИЕ:
    python bot_testing.py valera                    # Тестировать бота valera (все сценарии)
    python bot_testing.py valera final_scenarios    # Тестировать только файл final_scenarios.yaml
    python bot_testing.py valera -v                 # Подробный вывод
    python bot_testing.py valera --max-concurrent 10  # Увеличить количество потоков до 10 (По умолчанию 5)

ФАЙЛЫ СЦЕНАРИЕВ:
    bots/BOT_ID/tests/*.yaml - тестовые сценарии

ОТЧЕТЫ:
    bots/BOT_ID/reports/test_YYYYMMDD_HHMMSS.txt - подробные отчеты

ФОРМАТ СЦЕНАРИЕВ:
    expected_keywords поддерживает синонимы:
    - Одно слово: ["привет"] - проверяется только это слово
    - Синонимы: [["привет", "здравствуйте", "добро пожаловать"]] - достаточно найти любое из слов
    - Смешанный формат: ["привет", ["здравствуйте", "добро пожаловать"]] - комбинация одиночных слов и синонимов
"""

import argparse
import asyncio
import glob
import logging
import os
import sys
import time
import uuid
from datetime import datetime
from pathlib import Path
from typing import List

import yaml
from dotenv import load_dotenv
from project_root_finder import root

# Глобальная переменная для корневой директории проекта
PROJECT_ROOT = root  # smart_bot_factory/creation/ -> project_root


def set_project_root(new_root: Path):
    """Устанавливает новую корневую директорию проекта"""
    global PROJECT_ROOT
    PROJECT_ROOT = Path(new_root).resolve()
    logging.info(f"Корневая директория проекта изменена на: {PROJECT_ROOT}")


def get_project_root() -> Path:
    """Возвращает текущую корневую директорию проекта"""
    # Проверяем переменную окружения PROJECT_ROOT
    env_project_root = os.getenv("PROJECT_ROOT")
    if env_project_root:
        return Path(env_project_root).resolve()
    return PROJECT_ROOT


# Импорты для работы с ботом
try:
    # Попробуем относительные импорты (если запускается как модуль)
    from ..core.bot_utils import parse_ai_response
    from ..integrations.openai_client import OpenAIClient
    from ..integrations.supabase_client import SupabaseClient
    from ..utils.prompt_loader import PromptLoader
except ImportError:
    # Если не работает, используем абсолютные импорты (если запускается как скрипт)
    from smart_bot_factory.core.bot_utils import parse_ai_response
    from smart_bot_factory.integrations.openai_client import OpenAIClient
    from smart_bot_factory.integrations.supabase_client import SupabaseClient
    from smart_bot_factory.utils.prompt_loader import PromptLoader


logger = logging.getLogger(__name__)


class TestStep:
    """Класс для представления одного шага в сценарии"""

    def __init__(
        self,
        user_input: str,
        expected_keywords: List[str],
        forbidden_keywords: List[str] = None,
    ):
        self.user_input = user_input
        # Поддержка синонимов: если элемент списка - список, то это синонимы
        self.expected_keywords = self._process_keywords(expected_keywords)
        self.forbidden_keywords = [kw.lower() for kw in (forbidden_keywords or [])]

    def _process_keywords(self, keywords: List) -> List:
        """Обрабатывает ключевые слова, поддерживая синонимы"""
        processed = []
        for kw in keywords:
            if isinstance(kw, list):
                # Это группа синонимов
                synonyms = [s.lower() for s in kw if isinstance(s, str)]
                processed.append(synonyms)
            elif isinstance(kw, str):
                # Одно слово
                processed.append([kw.lower()])
            else:
                # Игнорируем неверные типы
                continue
        return processed


class TestScenario:
    """Класс для представления тестового сценария с последовательными шагами"""

    def __init__(self, name: str, steps: List[TestStep]):
        self.name = name
        self.steps = steps


class StepResult:
    """Класс для представления результата одного шага"""

    def __init__(
        self,
        step: TestStep,
        bot_response: str,
        passed: bool,
        missing_keywords: List[str] = None,
        found_forbidden: List[str] = None,
    ):
        self.step = step
        self.bot_response = bot_response
        self.passed = passed
        self.missing_keywords = missing_keywords or []
        self.found_forbidden = found_forbidden or []


class TestResult:
    """Класс для представления результата тестирования сценария"""

    def __init__(self, scenario: TestScenario, step_results: List[StepResult]):
        self.scenario = scenario
        self.step_results = step_results
        self.passed = all(step.passed for step in step_results)
        self.total_steps = len(step_results)
        self.passed_steps = sum(1 for step in step_results if step.passed)
        self.failed_steps = self.total_steps - self.passed_steps


class ScenarioLoader:
    """Загрузчик тестовых сценариев из YAML файлов"""

    @staticmethod
    def load_scenarios_from_file(file_path: str) -> List[TestScenario]:
        """Загружает сценарии из YAML файла"""
        try:
            with open(file_path, "r", encoding="utf-8") as file:
                data = yaml.safe_load(file)

            scenarios = []
            file_name = Path(file_path).stem

            for i, scenario_data in enumerate(data.get("scenarios", [])):
                # Проверяем новый формат со steps
                if "steps" in scenario_data:
                    # Новый формат: сценарий с шагами
                    name = scenario_data.get("name", f"[{file_name}-{i+1}]")
                    steps = []

                    for step_data in scenario_data["steps"]:
                        step = TestStep(
                            user_input=step_data.get("user_input", ""),
                            expected_keywords=step_data.get("expected_keywords", []),
                            forbidden_keywords=step_data.get("forbidden_keywords", []),
                        )
                        steps.append(step)

                    scenario = TestScenario(name=name, steps=steps)

                else:
                    # Старый формат: одиночный вопрос -> превращаем в сценарий с одним шагом
                    name = scenario_data.get("name", f"[{file_name}-{i+1}]")
                    step = TestStep(
                        user_input=scenario_data.get("user_input", ""),
                        expected_keywords=scenario_data.get("expected_keywords", []),
                        forbidden_keywords=scenario_data.get("forbidden_keywords", []),
                    )
                    scenario = TestScenario(name=name, steps=[step])

                # Добавляем информацию о файле для отчетности
                scenario.source_file = file_name
                scenario.scenario_number = i + 1

                scenarios.append(scenario)

            return scenarios

        except Exception as e:
            logging.error(f"Ошибка загрузки сценариев из {file_path}: {e}")
            return []

    @staticmethod
    def load_all_scenarios_for_bot(
        bot_id: str, project_root: Path = None
    ) -> List[TestScenario]:
        """Загружает все сценарии для указанного бота"""
        # Используем переданную корневую директорию или глобальную
        user_root_dir = project_root or get_project_root()

        # Проверяем наличие папки bots/{bot_id}
        bots_dir = user_root_dir / "bots" / bot_id
        if not bots_dir.exists():
            logging.warning(f"Папка bots/{bot_id} не найдена в проекте: {bots_dir}")
            return []

        # Путь к папке tests в корневой директории пользователя
        tests_dir = user_root_dir / "bots" / bot_id / "tests"

        if not tests_dir.exists():
            logging.warning(f"Каталог тестов не найден: {tests_dir}")
            return []

        all_scenarios = []
        for yaml_file in tests_dir.glob("*.yaml"):
            scenarios = ScenarioLoader.load_scenarios_from_file(str(yaml_file))
            all_scenarios.extend(scenarios)

        return all_scenarios


class BotTester:
    """Основной класс для тестирования ботов"""

    def __init__(self, bot_id: str, project_root: Path = None):
        self.bot_id = bot_id
        self.project_root = project_root or get_project_root()
        self.openai_client = None
        self.prompt_loader = None
        self.supabase_client = None
        self.config_dir = None
        self._initialize_bot()

    def _initialize_bot(self):
        """Инициализация компонентов бота, используя готовую логику из запускалки"""
        try:
            logging.info("")
            logging.info("══════════════════════════════════════════════════════════")
            logging.info(f"ИНИЦИАЛИЗАЦИЯ ТЕСТЕРА БОТА: {self.bot_id}")
            logging.info("══════════════════════════════════════════════════════════")

            # Используем корневую директорию проекта
            user_root_dir = self.project_root
            logging.info(f"Корневая директория проекта: {user_root_dir}")

            # Добавляем корневую директорию в sys.path для импорта библиотеки
            sys.path.insert(0, str(user_root_dir))
            logging.info(f"Добавлен путь в sys.path: {user_root_dir}")

            # Путь к конфигурации бота в корневой директории проекта
            self.config_dir = user_root_dir / "bots" / self.bot_id
            logging.info(f"Каталог конфигурации: {self.config_dir}")

            if not self.config_dir.exists():
                raise ValueError(f"Папка конфигурации не найдена: {self.config_dir}")
            logging.info("Каталог конфигурации найден")

            # Устанавливаем BOT_ID как в оригинальном файле
            os.environ["BOT_ID"] = self.bot_id
            logging.info(f"BOT_ID установлен: {self.bot_id}")

            # Загружаем .env из папки бота
            env_file = self.config_dir / ".env"
            logging.info(f"Загрузка .env файла: {env_file}")
            if env_file.exists():
                load_dotenv(env_file)
                logging.info("⚙️ Чтение конфигурации из .env...")

            # OpenAI настройки
            openai_api_key = os.getenv("OPENAI_API_KEY")
            openai_model = os.getenv("OPENAI_MODEL", "gpt-4o-mini")
            openai_max_tokens = int(os.getenv("OPENAI_MAX_TOKENS", "4000"))
            openai_temperature = float(os.getenv("OPENAI_TEMPERATURE", "0.7"))

            # Supabase настройки
            supabase_url = os.getenv("SUPABASE_URL")
            supabase_key = os.getenv("SUPABASE_KEY")

            # Проверяем обязательные переменные
            if not openai_api_key:
                raise ValueError("OPENAI_API_KEY не найден в .env файле")
            if not supabase_url:
                raise ValueError("SUPABASE_URL не найден в .env файле")
            if not supabase_key:
                raise ValueError("SUPABASE_KEY не найден в .env файле")

            logging.info(f"Конфигурация загружена (модель: {openai_model})")

            logging.info("Инициализация OpenAI клиента...")
            self.openai_client = OpenAIClient(
                api_key=openai_api_key,
                model=openai_model,
                max_tokens=openai_max_tokens,
                temperature=openai_temperature,
            )
            logging.info(f"OpenAI клиент создан (модель: {openai_model})")

            logging.info("Инициализация загрузчика промптов...")
            # PromptLoader автоматически найдет все .txt файлы в папке промптов
            self.prompt_loader = PromptLoader(prompts_dir=self.config_dir / "prompts")
            logging.info("Загрузчик промптов создан")

            # Инициализируем Supabase клиент
            logging.info("Инициализация Supabase клиента...")
            self.supabase_client = SupabaseClient(
                url=supabase_url, key=supabase_key, bot_id=self.bot_id
            )
            logging.info("Supabase клиент создан")

            logging.info(f"Бот {self.bot_id} инициализирован успешно!")
            logging.info("══════════════════════════════════════════════════════════")

        except Exception as e:
            logging.error("══════════════════════════════════════════════════════════")
            logging.error(f"ОШИБКА ИНИЦИАЛИЗАЦИИ БОТА: {self.bot_id}")
            logging.error(f"Описание: {e}")
            logging.error("══════════════════════════════════════════════════════════")
            raise

    async def test_scenario(self, scenario: TestScenario) -> TestResult:
        """Тестирует сценарий с последовательными шагами"""
        try:
            # Инициализируем Supabase клиент перед тестом
            if not self.supabase_client.client:
                await self.supabase_client.initialize()
                logging.info(
                    f"🔌 Supabase клиент инициализирован для бота {self.bot_id}"
                )

            step_results = []

            # Генерируем уникальный тестовый telegram_id (только цифры)
            # Формат: 999 + timestamp + случайные цифры -> гарантированно уникальный
            timestamp_part = str(int(time.time()))[-6:]  # Последние 6 цифр timestamp
            random_part = str(uuid.uuid4().int)[:3]  # Первые 3 цифры из UUID
            unique_test_telegram_id = int(f"999{timestamp_part}{random_part}")

            user_data = {
                "telegram_id": unique_test_telegram_id,
                "username": "test_user",
                "first_name": "Test",
                "last_name": "User",
                "language_code": "ru",
            }

            logging.info("")
            logging.info(
                "🧪 ═══════════════════════════════════════════════════════════"
            )
            logging.info(f"🎯 НАЧИНАЕМ ТЕСТ СЦЕНАРИЯ: {scenario.name}")
            logging.info(f"🤖 Бот: {self.bot_id}")
            logging.info(f"👤 Тестовый пользователь: {unique_test_telegram_id}")
            logging.info(f"📝 Количество шагов: {len(scenario.steps)}")
            logging.info("═══════════════════════════════════════════════════════════")

            session_id, system_prompt = await self.create_test_session(user_data)
            logging.info(f"🆔 Создана тестовая сессия: {session_id}")

            for i, step in enumerate(scenario.steps):
                step_num = i + 1
                logging.info("")
                logging.info(
                    f"🔄 ─────────────── ШАГ {step_num}/{len(scenario.steps)} ───────────────"
                )
                logging.info(f"💬 Ввод пользователя: '{step.user_input}'")

                if step.expected_keywords:
                    # Форматируем ожидаемые ключевые слова для логов
                    expected_display = []
                    for group in step.expected_keywords:
                        if len(group) == 1:
                            expected_display.append(group[0])
                        else:
                            expected_display.append(f"[{'/'.join(group)}]")
                    logging.info(f"🎯 Ожидаемые слова: {expected_display}")
                if step.forbidden_keywords:
                    logging.info(f"🚫 Запрещенные слова: {step.forbidden_keywords}")

                # Анализируем ответ на этом шаге
                start_time = time.time()
                clean_response = await self.process_user_message_test(
                    step.user_input, session_id, system_prompt
                )
                step_duration = int((time.time() - start_time) * 1000)

                # Показываем ответ бота (обрезанный)
                response_preview = (
                    clean_response[:150] + "..."
                    if len(clean_response) > 150
                    else clean_response
                )
                response_preview = response_preview.replace("\n", " ")
                logging.info(f"🤖 Ответ бота: '{response_preview}'")
                logging.info(f"⏱️ Время обработки: {step_duration}мс")

                # Проверяем ожидаемые ключевые слова (с поддержкой синонимов)
                missing_keyword_groups = []
                found_expected = []
                for keyword_group in step.expected_keywords:
                    # keyword_group - это либо список синонимов, либо список с одним словом
                    found_in_group = False
                    found_synonym = None

                    for synonym in keyword_group:
                        if synonym in clean_response.lower():
                            found_in_group = True
                            found_synonym = synonym
                            break

                    if found_in_group:
                        found_expected.append(found_synonym)
                    else:
                        # Если группа синонимов не найдена, добавляем всю группу в missing
                        missing_keyword_groups.append(keyword_group)

                # Проверяем запрещенные ключевые слова
                found_forbidden = []
                for keyword in step.forbidden_keywords:
                    if keyword.lower() in clean_response.lower():
                        found_forbidden.append(keyword)

                # Выводим результаты проверки
                if found_expected:
                    logging.info(f"✅ Найденные ожидаемые: {found_expected}")
                if missing_keyword_groups:
                    # Показываем пропущенные группы синонимов в удобном формате
                    missing_display = []
                    for group in missing_keyword_groups:
                        if len(group) == 1:
                            missing_display.append(group[0])
                        else:
                            missing_display.append(f"[{'/'.join(group)}]")
                    logging.info(f"❌ НЕ найденные ожидаемые: {missing_display}")
                if found_forbidden:
                    logging.info(f"🚫 Найденные запрещенные: {found_forbidden}")

                # Определяем результат шага
                passed = len(missing_keyword_groups) == 0 and len(found_forbidden) == 0
                status_icon = "✅" if passed else "❌"
                status_text = "ПРОЙДЕН" if passed else "ПРОВАЛЕН"
                logging.info(
                    f"🎯 Результат шага {step_num}: {status_icon} {status_text}"
                )

                # Преобразуем missing_keyword_groups в плоский список для обратной совместимости
                missing_keywords_flat = []
                for group in missing_keyword_groups:
                    missing_keywords_flat.extend(group)

                step_result = StepResult(
                    step=step,
                    bot_response=clean_response,
                    passed=passed,
                    missing_keywords=missing_keywords_flat,
                    found_forbidden=found_forbidden,
                )

                step_results.append(step_result)

                # Короткая пауза между шагами
                await asyncio.sleep(0.1)

            # Финальная статистика сценария
            passed_steps = sum(1 for step in step_results if step.passed)
            total_steps = len(step_results)
            success_rate = (passed_steps / total_steps) * 100 if total_steps > 0 else 0

            logging.info("")
            logging.info("🏁 ─────────────── ИТОГ СЦЕНАРИЯ ───────────────")
            logging.info(
                f"📊 Пройдено шагов: {passed_steps}/{total_steps} ({success_rate:.1f}%)"
            )
            overall_status = (
                "✅ СЦЕНАРИЙ ПРОЙДЕН"
                if passed_steps == total_steps
                else "❌ СЦЕНАРИЙ ПРОВАЛЕН"
            )
            logging.info(f"🎯 {overall_status}")
            logging.info("═══════════════════════════════════════════════════════════")

            return TestResult(scenario=scenario, step_results=step_results)

        except Exception as e:
            logging.error("")
            logging.error(
                "💥 ═══════════════════════════════════════════════════════════"
            )
            logging.error(f"❌ КРИТИЧЕСКАЯ ОШИБКА В СЦЕНАРИИ: {scenario.name}")
            logging.error(f"🐛 Ошибка: {str(e)}")
            logging.error(
                "═══════════════════════════════════════════════════════════"
            )
            logging.exception("📋 Полный стек ошибки:")

            # Создаем результат с ошибкой для всех шагов
            step_results = []
            for step in scenario.steps:
                step_result = StepResult(
                    step=step,
                    bot_response=f"ОШИБКА: {str(e)}",
                    passed=False,
                    missing_keywords=step.expected_keywords,
                    found_forbidden=[],
                )
                step_results.append(step_result)

            return TestResult(scenario=scenario, step_results=step_results)

    async def get_welcome_file_caption_test(self) -> str:
        """
        Тестовая версия получения подписи к приветственному файлу.
        Возвращает только текст из файла welcome_file_msg.txt без взаимодействия с ботом.

        Returns:
            str: текст подписи из файла или пустая строка
        """
        try:
            # Путь к папке welcome_files (абсолютный путь)
            folder = self.config_dir / "welcome_files"
            if not folder.exists() or not folder.is_dir():
                logger.info(f"Директория приветственных файлов не найдена: {folder}")
                return ""

            # Ищем файл welcome_file_msg.txt в директории
            msg_path = folder / "welcome_file_msg.txt"
            if not msg_path.is_file():
                logger.info(f"Файл подписи не найден: {msg_path}")
                return ""

            # Читаем содержимое файла
            try:
                with open(msg_path, "r", encoding="utf-8") as f:
                    caption = f.read().strip()
                    logger.info(f"Подпись загружена из файла: {msg_path}")
                    return caption
            except Exception as e:
                logger.error(f"Ошибка при чтении файла подписи {msg_path}: {e}")
                return ""

        except Exception as e:
            logger.error(f"Ошибка при получении подписи к приветственному файлу: {e}")
            return ""

    async def create_test_session(self, user_data: dict) -> tuple[str, str]:
        """
        Создает тестовую сессию без взаимодействия с ботом.

        Args:
            user_data: словарь с данными пользователя (telegram_id, username, first_name, last_name, language_code)

        Returns:
            tuple[str, str, str]: (session_id, system_prompt, welcome_message)
            - session_id: ID созданной сессии
            - system_prompt: системный промпт
            - welcome_message: приветственное сообщение (включая подпись к файлу, если есть)
        """

        try:
            logging.info("🔄 Создание тестовой сессии...")

            # 1. ЗАГРУЖАЕМ ПРОМПТЫ
            logging.info("📋 Загрузка промптов...")
            system_prompt = await self.prompt_loader.load_system_prompt()
            welcome_message = await self.prompt_loader.load_welcome_message()
            logging.info(
                f"✅ Промпты загружены: система ({len(system_prompt)} симв.), приветствие ({len(welcome_message)} симв.)"
            )

            # 2. СОЗДАЕМ НОВУЮ СЕССИЮ
            logging.info("🗄️ Создание сессии в Supabase...")
            session_id = await self.supabase_client.create_chat_session(
                user_data, system_prompt
            )
            logging.info(f"✅ Сессия создана с ID: {session_id}")

            # 3. ПРОВЕРЯЕМ НАЛИЧИЕ ПРИВЕТСТВЕННОГО ФАЙЛА И ЕГО ПОДПИСИ
            logging.info("📎 Проверка приветственного файла...")
            caption = await self.get_welcome_file_caption_test()

            # 4. ОБЪЕДИНЯЕМ ПРИВЕТСТВЕННОЕ СООБЩЕНИЕ С ПОДПИСЬЮ К ФАЙЛУ
            if caption:
                welcome_message = f"{welcome_message}\n\nПодпись к файлу:\n\n{caption}"
                logging.info(f"📎 Добавлена подпись к файлу ({len(caption)} симв.)")
            else:
                logging.info("📎 Приветственный файл не найден или пуст")

            # 5. СОХРАНЯЕМ ПРИВЕТСТВЕННОЕ СООБЩЕНИЕ В БД
            logging.info("💾 Сохранение приветственного сообщения в БД...")
            await self.supabase_client.add_message(
                session_id=session_id,
                role="assistant",
                content=welcome_message,
                message_type="text",
            )
            logging.info("✅ Приветственное сообщение сохранено")

            return session_id, system_prompt

        except Exception as e:
            logging.error(f"💥 Ошибка в create_test_session: {e}")
            logging.exception("📋 Полный стек ошибки:")
            raise

    async def process_user_message_test(
        self, user_message: str, session_id: str, system_prompt: str
    ):
        """
        Тестовая версия обработки сообщений пользователя без взаимодействия с ботом.
        Возвращает только текст ответа от нейросети и сохраняет данные в БД.
        """

        import time
        from datetime import datetime

        import pytz

        # Читаем настройки напрямую из .env
        max_context_messages = int(os.getenv("MAX_CONTEXT_MESSAGES", "20"))

        try:
            logging.info("📨 Обработка сообщения пользователя...")

            # Сохраняем сообщение пользователя
            logging.info("💾 Сохранение сообщения пользователя в БД...")
            await self.supabase_client.add_message(
                session_id=session_id,
                role="user",
                content=user_message,
                message_type="text",
            )
            logging.info("✅ Сообщение пользователя сохранено")

            # Получаем историю сообщений
            logging.info(
                f"📚 Получение истории сообщений (лимит: {max_context_messages})..."
            )
            chat_history = await self.supabase_client.get_chat_history(
                session_id, limit=max_context_messages
            )
            logging.info(f"✅ Получено {len(chat_history)} сообщений из истории")

            # Добавляем время
            moscow_tz = pytz.timezone("Europe/Moscow")
            current_time = datetime.now(moscow_tz)
            time_info = current_time.strftime("%H:%M, %d.%m.%Y, %A")
            logging.info(f"🕐 Текущее время: {time_info}")

            # Модифицируем системный промпт с временем
            system_prompt_with_time = f"""
    {system_prompt}

    ТЕКУЩЕЕ ВРЕМЯ: {time_info} (московское время)
    """

            # Формируем контекст для OpenAI
            messages = [{"role": "system", "content": system_prompt_with_time}]

            for msg in chat_history[-max_context_messages:]:
                messages.append({"role": msg["role"], "content": msg["content"]})

            # Добавляем финальные инструкции
            logging.info("📋 Загрузка финальных инструкций...")
            final_instructions = await self.prompt_loader.load_final_instructions()
            if final_instructions:
                messages.append({"role": "system", "content": final_instructions})
                logging.info(
                    f"✅ Финальные инструкции добавлены ({len(final_instructions)} симв.)"
                )
            else:
                logging.info("📋 Финальные инструкции отсутствуют")

            logging.info(
                f"🧠 Отправка запроса к OpenAI (сообщений в контексте: {len(messages)})..."
            )

            # Получаем ответ от OpenAI
            start_time = time.time()
            ai_response = await self.openai_client.get_completion(messages)
            processing_time = int((time.time() - start_time) * 1000)

            logging.info(f"✅ Ответ получен от OpenAI за {processing_time}мс")

            # Инициализируем переменные
            tokens_used = 0
            ai_metadata = {}
            response_text = ""

            # Проверяем ответ
            if not ai_response or not ai_response.strip():
                response_text = "Извините, произошла техническая ошибка. Попробуйте переформулировать вопрос."
                tokens_used = 0
                ai_metadata = {}
                logging.warning("⚠️ Пустой ответ от OpenAI")
            else:
                tokens_used = self.openai_client.estimate_tokens(ai_response)
                logging.info(f"🔢 Оценка токенов: {tokens_used}")

                logging.info("🔍 Парсинг AI-ответа на метаданные...")
                response_text, ai_metadata = parse_ai_response(ai_response)

                if not ai_metadata:
                    response_text = ai_response
                    ai_metadata = {}
                    logging.info("📝 Метаданные не найдены, используем исходный ответ")
                elif not response_text.strip():
                    response_text = ai_response
                    logging.info(
                        "📝 Пустой текст после парсинга, используем исходный ответ"
                    )
                else:
                    logging.info(f"📝 Найдены метаданные: {list(ai_metadata.keys())}")

                # Обновляем этап сессии и качество лида если есть метаданные
                if ai_metadata:
                    stage = ai_metadata.get("этап")
                    quality = ai_metadata.get("качество")
                    if stage or quality is not None:
                        logging.info(
                            f"🎯 Обновление этапа сессии: этап={stage}, качество={quality}"
                        )
                        await self.supabase_client.update_session_stage(
                            session_id, stage, quality
                        )

            # Сохраняем ответ ассистента
            logging.info("💾 Сохранение ответа ассистента в БД...")
            await self.supabase_client.add_message(
                session_id=session_id,
                role="assistant",
                content=response_text,
                message_type="text",
                tokens_used=tokens_used,
                processing_time_ms=processing_time,
                ai_metadata=ai_metadata,
            )
            logging.info("✅ Ответ ассистента сохранен")

            return response_text

        except Exception as e:
            logging.error(f"💥 Ошибка в process_user_message_test: {e}")
            logging.exception("📋 Полный стек ошибки:")
            return "Произошла ошибка при обработке сообщения."


class TestRunner:
    """Запускатель тестов с асинхронным выполнением"""

    def __init__(self, bot_id: str, max_concurrent: int = 5, project_root: Path = None):
        self.bot_id = bot_id
        self.max_concurrent = max_concurrent
        self.project_root = project_root or get_project_root()
        self.bot_tester = BotTester(bot_id, self.project_root)

    async def run_tests(self, scenarios: List[TestScenario]) -> List[TestResult]:
        """Запускает тесты асинхронно"""
        semaphore = asyncio.Semaphore(self.max_concurrent)

        async def run_single_test(scenario: TestScenario) -> TestResult:
            async with semaphore:
                logging.info(f"🧪 Выполнение теста: {scenario.name}")
                result = await self.bot_tester.test_scenario(scenario)
                status = "✅" if result.passed else "❌"
                logging.info(f"   {status} {scenario.name}")
                return result

        # Запускаем все тесты параллельно
        tasks = [run_single_test(scenario) for scenario in scenarios]
        results = await asyncio.gather(*tasks, return_exceptions=True)

        # Обрабатываем исключения
        processed_results = []
        for i, result in enumerate(results):
            if isinstance(result, Exception):
                logging.error(f"Исключение в тесте {scenarios[i].name}: {result}")
                # Создаем результат с ошибкой для всех шагов
                step_results = []
                for step in scenarios[i].steps:
                    step_result = StepResult(
                        step=step,
                        bot_response=f"ИСКЛЮЧЕНИЕ: {str(result)}",
                        passed=False,
                        missing_keywords=step.expected_keywords,
                        found_forbidden=[],
                    )
                    step_results.append(step_result)

                processed_results.append(
                    TestResult(scenario=scenarios[i], step_results=step_results)
                )
            else:
                processed_results.append(result)

        return processed_results


class ReportGenerator:
    """Генератор отчетов о тестировании"""

    @staticmethod
    def cleanup_old_reports(reports_dir: str, max_reports: int = 10):
        """Удаляет старые отчеты, оставляя только max_reports самых новых"""
        if not os.path.exists(reports_dir):
            return

        # Находим все файлы отчетов
        report_pattern = os.path.join(reports_dir, "test_*.txt")
        report_files = glob.glob(report_pattern)

        if len(report_files) <= max_reports:
            return  # Количество файлов в пределах лимита

        # Сортируем по времени модификации (старые первыми)
        report_files.sort(key=lambda x: os.path.getmtime(x))

        # Удаляем старые файлы, оставляя только max_reports-1 (место для нового)
        files_to_delete = report_files[: -(max_reports - 1)]

        for file_path in files_to_delete:
            try:
                os.remove(file_path)
                filename = os.path.basename(file_path)
                logging.info(f"🗑️ Удален старый отчет: {filename}")
            except Exception as e:
                logging.warning(f"Не удалось удалить отчет {file_path}: {e}")

    @staticmethod
    def generate_console_report(bot_id: str, results: List[TestResult]):
        """Генерирует отчет в консоль"""
        passed_count = sum(1 for r in results if r.passed)
        # failed_count = len(results) - passed_count  # Не используется
        total_steps = sum(r.total_steps for r in results)
        passed_steps = sum(r.passed_steps for r in results)
        success_rate = (passed_count / len(results)) * 100 if results else 0
        step_success_rate = (passed_steps / total_steps) * 100 if total_steps else 0

        print(f"\n📊 РЕЗУЛЬТАТЫ: {bot_id.upper()}")
        print(
            f"✅ Сценариев пройдено: {passed_count}/{len(results)} ({success_rate:.1f}%)"
        )
        print(
            f"📝 Шагов пройдено: {passed_steps}/{total_steps} ({step_success_rate:.1f}%)"
        )

        # Определяем уровень качества по сценариям
        if success_rate >= 90:
            print("🎉 ОТЛИЧНО — бот готов к продакшену")
        elif success_rate >= 80:
            print("✅ ХОРОШО — небольшие улучшения")
        elif success_rate >= 60:
            print("⚠️ УДОВЛЕТВОРИТЕЛЬНО — требуются улучшения")
        else:
            print("🚨 ПЛОХО — критические проблемы с промптами")

    @staticmethod
    def generate_detailed_report(bot_id: str, results: List[TestResult]) -> str:
        """Генерирует подробный отчет"""
        timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")

        report_lines = [
            f"ОТЧЕТ ТЕСТИРОВАНИЯ: {bot_id.upper()}",
            f"Время: {timestamp}",
            f"Сценариев: {len(results)}",
            "",
        ]

        # Статистика
        passed_count = sum(1 for r in results if r.passed)
        total_steps = sum(r.total_steps for r in results)
        passed_steps = sum(r.passed_steps for r in results)
        success_rate = (passed_count / len(results)) * 100 if results else 0
        step_success_rate = (passed_steps / total_steps) * 100 if total_steps else 0

        report_lines.extend(
            [
                f"УСПЕШНОСТЬ СЦЕНАРИЕВ: {success_rate:.1f}% ({passed_count}/{len(results)})",
                f"УСПЕШНОСТЬ ШАГОВ: {step_success_rate:.1f}% ({passed_steps}/{total_steps})",
                "",
            ]
        )

        # Список ошибок в требуемом формате
        failed_tests = [r for r in results if not r.passed]
        if failed_tests:
            report_lines.extend(
                [
                    "═══════════════════════════════════════════════════════════════",
                    "СПИСОК ОШИБОК:",
                    "═══════════════════════════════════════════════════════════════",
                ]
            )

            for result in failed_tests:
                scenario = result.scenario
                source_file = getattr(scenario, "source_file", "unknown")
                scenario_number = getattr(scenario, "scenario_number", "?")

                # Правильное отображение имени сценария
                display_name = (
                    scenario.name
                    if not scenario.name.startswith("[")
                    else f"[{source_file}-{scenario_number}]"
                )
                report_lines.extend(
                    [
                        f"ФАЙЛ: {source_file}.yaml | СЦЕНАРИЙ: {display_name}",
                        f"СТАТУС: {result.passed_steps}/{result.total_steps} шагов пройдено",
                        "",
                    ]
                )

                # Детальная информация о каждом шаге
                for i, step_result in enumerate(result.step_results):
                    step_num = i + 1
                    status = "✅" if step_result.passed else "❌"

                    # Форматируем ожидаемые ключевые слова с учетом синонимов
                    expected_display = []
                    for group in step_result.step.expected_keywords:
                        if len(group) == 1:
                            expected_display.append(group[0])
                        else:
                            expected_display.append(f"[{'/'.join(group)}]")

                    report_lines.extend(
                        [
                            f"ШАГ {step_num} {status}:",
                            f'  Ввод: "{step_result.step.user_input}"',
                            f"  Ожидаемые: {expected_display}",
                            f"  Запрещенные: {step_result.step.forbidden_keywords}",
                            "",
                        ]
                    )

                    # Полный ответ бота для всех шагов (и успешных, и провальных)
                    bot_response = step_result.bot_response.strip()
                    # Заменяем переносы строк на символы для лучшей читаемости в отчете
                    bot_response_formatted = bot_response.replace("\n", "\\n")

                    report_lines.extend(
                        ["  Полный ответ бота:", f'  "{bot_response_formatted}"', ""]
                    )

                    # Конкретные ошибки для неуспешных шагов
                    if not step_result.passed:
                        if step_result.missing_keywords:
                            report_lines.append(
                                f"    ❌ НЕ НАЙДЕНЫ: {', '.join(step_result.missing_keywords)}"
                            )

                        if step_result.found_forbidden:
                            report_lines.append(
                                f"    ❌ НАЙДЕНЫ ЗАПРЕЩЕННЫЕ: {', '.join(step_result.found_forbidden)}"
                            )

                        report_lines.append("")

                report_lines.extend(["-" * 67, ""])

        # Краткая сводка по пройденным тестам
        passed_tests = [r for r in results if r.passed]
        if passed_tests:
            report_lines.extend(["ПРОЙДЕННЫЕ СЦЕНАРИИ:", ""])
            for result in passed_tests:
                scenario = result.scenario
                source_file = getattr(scenario, "source_file", "unknown")
                scenario_number = getattr(scenario, "scenario_number", "?")

                # Правильное отображение имени сценария для пройденных тестов (сокращенный формат)
                display_name = (
                    scenario.name
                    if not scenario.name.startswith("[")
                    else f"[{source_file}-{scenario_number}]"
                )
                report_lines.append(f"✅ {source_file}.yaml | {display_name}")

        return "\n".join(report_lines)

    @staticmethod
    def save_report(
        bot_id: str, results: List[TestResult], project_root: Path = None
    ) -> str:
        """Сохраняет отчет в файл"""
        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")

        # Используем переданную корневую директорию или глобальную
        user_root_dir = project_root or get_project_root()

        # Проверяем наличие папки bots/{bot_id}
        bots_dir = user_root_dir / "bots" / bot_id
        if not bots_dir.exists():
            logging.warning(f"Папка bots/{bot_id} не найдена в проекте: {bots_dir}")
            return ""

        # Создаем папку отчетов в папке бота
        reports_dir = user_root_dir / "bots" / bot_id / "reports"
        os.makedirs(reports_dir, exist_ok=True)

        # Очищаем старые отчеты перед созданием нового
        ReportGenerator.cleanup_old_reports(str(reports_dir), max_reports=10)

        # Лаконичное название файла
        report_filename = reports_dir / f"test_{timestamp}.txt"

        report_content = ReportGenerator.generate_detailed_report(bot_id, results)

        with open(report_filename, "w", encoding="utf-8") as f:
            f.write(report_content)

        return str(report_filename)


async def main():
    """Главная функция CLI"""
    parser = argparse.ArgumentParser(description="Система тестирования ботов")
    parser.add_argument(
        "bot_id",
        nargs="?",
        default="growthmed-october-24",
        help="ID бота для тестирования",
    )
    parser.add_argument(
        "scenario_file",
        nargs="?",
        default=None,
        help="Название файла сценариев (без расширения или с .yaml)",
    )
    parser.add_argument(
        "--scenario-file",
        dest="scenario_file_legacy",
        help="Конкретный файл сценариев (устаревший параметр)",
    )
    parser.add_argument(
        "--max-concurrent",
        type=int,
        default=5,
        help="Максимальное количество параллельных тестов",
    )
    parser.add_argument("--verbose", "-v", action="store_true", help="Подробный вывод")

    args = parser.parse_args()

    # Настройка логирования
    log_level = logging.INFO if args.verbose else logging.INFO  # Всегда показываем INFO
    logging.basicConfig(level=log_level, format="%(message)s")

    try:
        # Определяем какой файл сценариев использовать
        scenario_file = args.scenario_file or args.scenario_file_legacy

        # Получаем корневую директорию проекта
        project_root = get_project_root()

        # Загружаем сценарии
        if scenario_file:
            # Обрабатываем название файла
            if not scenario_file.endswith(".yaml"):
                scenario_file += ".yaml"

            # Проверяем наличие папки bots/{bot_id}
            bots_dir = project_root / "bots" / args.bot_id
            if not bots_dir.exists():
                print(f"Папка bots/{args.bot_id} не найдена в проекте: {bots_dir}")
                return 1

            # Путь к файлу сценариев в корневой директории проекта
            scenario_path = (
                project_root / "bots" / args.bot_id / "tests" / scenario_file
            )
            scenarios = ScenarioLoader.load_scenarios_from_file(str(scenario_path))

            if not scenarios:
                print(f"Файл сценариев '{scenario_file}' не найден или пуст")
                return 1
        else:
            scenarios = ScenarioLoader.load_all_scenarios_for_bot(
                args.bot_id, project_root
            )

            if not scenarios:
                print(f"Сценарии для бота '{args.bot_id}' не найдены")
                return 1

        print(f"🚀 Запуск тестирования бота: {args.bot_id}")
        if scenario_file:
            print(f"📋 Тестируется файл: {scenario_file}")
        else:
            print("📋 Тестируются все файлы сценариев")
        print(f"📋 Найдено сценариев: {len(scenarios)}")

        # Запускаем тесты
        test_runner = TestRunner(args.bot_id, args.max_concurrent, project_root)
        results = await test_runner.run_tests(scenarios)

        # Генерируем отчеты
        ReportGenerator.generate_console_report(args.bot_id, results)
        report_file = ReportGenerator.save_report(args.bot_id, results, project_root)

        print(f"\n📄 Подробный отчет сохранен: {report_file}")

        # Возвращаем код выхода
        failed_count = sum(1 for r in results if not r.passed)
        return 0 if failed_count == 0 else 1

    except Exception as e:
        logging.error(f"Критическая ошибка: {e}")
        return 1


if __name__ == "__main__":
    try:
        exit_code = asyncio.run(main())
        sys.exit(exit_code)
    except KeyboardInterrupt:
        print("\n⏹️ Тестирование прервано пользователем")
        sys.exit(130)



from .common import get_user_locale
from .dialog_calendar import DialogCalendar
from .schemas import (CalendarLabels, DialogCalendarCallback,
                      SimpleCalendarCallback)
from .simple_calendar import SimpleCalendar

__all__ = [
    "SimpleCalendar",
    "DialogCalendar",
    "SimpleCalendarCallback",
    "DialogCalendarCallback",
    "CalendarLabels",
    "get_user_locale",
]



# Исправленный conversation_manager.py после обновления из GitHub - фикс расчета времени до автозавершения

import logging
from datetime import datetime, timezone
from typing import Any, Dict, List, Optional

from aiogram.fsm.context import FSMContext
from aiogram.types import Message, User

from .bot_utils import get_global_var

logger = logging.getLogger(__name__)


class ConversationManager:
    """Управление диалогами между админами и пользователями"""

    def __init__(
        self, supabase_client, admin_manager, parse_mode, admin_session_timeout_minutes
    ):
        self.supabase = supabase_client
        self.admin_manager = admin_manager
        self.parse_mode = parse_mode
        self.admin_session_timeout_minutes = admin_session_timeout_minutes

    async def start_admin_conversation(self, admin_id: int, user_id: int) -> bool:
        """Начинает диалог админа с пользователем"""
        try:
            from ..utils.debug_routing import debug_admin_conversation_creation

            await debug_admin_conversation_creation(admin_id, user_id)

            # Проверяем, что это действительно админ
            if not self.admin_manager.is_admin(admin_id):
                logger.warning(f"Попытка начать диалог не-админом {admin_id}")
                return False

            # Получаем активную сессию пользователя
            session_info = await self.supabase.get_active_session(user_id)
            if not session_info:
                logger.warning(f"У пользователя {user_id} нет активной сессии")
                return False

            session_id = session_info["id"]
            logger.info(f"✅ Найдена активная сессия: {session_id}")

            # Создаем запись о диалоге в БД
            logger.info("🔧 Создаем запись о диалоге в БД...")
            conversation_id = await self.supabase.start_admin_conversation(
                admin_id, user_id, session_id
            )
            logger.info(f"✅ Диалог создан с ID: {conversation_id}")

            # Показываем последние 5 сообщений
            await self._show_recent_messages(admin_id, user_id, session_id)

            logger.info(
                f"🎉 Диалог успешно начат: админ {admin_id} -> пользователь {user_id}"
            )
            return True

        except Exception as e:
            logger.error(
                f"❌ Ошибка начала диалога админа {admin_id} с пользователем {user_id}: {e}"
            )
            logger.exception("Полный стек ошибки:")
            return False

    async def _show_recent_messages(self, admin_id: int, user_id: int, session_id: str):
        """Показывает последние сообщения пользователя"""
        from .message_sender import send_message_by_human

        try:
            # Получаем последние 5 сообщений (сортируем по убыванию и берем первые 5)
            query = (
                self.supabase.client.table("sales_messages")
                .select("role", "content", "created_at")
                .eq("session_id", session_id)
                .order("created_at", desc=True)
                .limit(5)
            )

            # Добавляем фильтр по bot_id если он указан
            if self.supabase.bot_id:
                query = query.eq("bot_id", self.supabase.bot_id)

            response = query.execute()

            recent_messages = response.data if response.data else []

            if not recent_messages:
                await send_message_by_human(
                    admin_id, "📭 Нет сообщений в текущей сессии"
                )
                return

            # Получаем красивое имя пользователя
            user_display = await self.get_user_display_name(user_id)

            header = f"📜 Последние сообщения с {user_display}\n{'━' * 40}"
            await send_message_by_human(admin_id, header)

            # Разворачиваем чтобы показать в хронологическом порядке (старые -> новые)
            for msg in reversed(recent_messages):
                role_emoji = "👤" if msg["role"] == "user" else "🤖"
                timestamp = datetime.fromisoformat(
                    msg["created_at"].replace("Z", "+00:00")
                )
                time_str = timestamp.strftime("%H:%M")

                # Сокращаем длинные сообщения
                content = self._truncate_message(msg["content"])

                message_text = f"{role_emoji} {time_str} | {content}"

                await send_message_by_human(admin_id, message_text)

            # Разделитель
            await send_message_by_human(
                admin_id,
                f"{'━' * 40}\n💬 Диалог начат. Ваши сообщения будут переданы пользователю.",
            )

        except Exception as e:
            logger.error(f"Ошибка показа последних сообщений: {e}")

    async def get_user_display_name(self, user_id: int) -> str:
        """Получает красивое отображение пользователя с username"""
        try:
            query = (
                self.supabase.client.table("sales_users")
                .select("first_name", "last_name", "username")
                .eq("telegram_id", user_id)
            )
            
            # Добавляем фильтр по bot_id если он указан
            if self.supabase.bot_id:
                query = query.eq("bot_id", self.supabase.bot_id)
            
            response = query.execute()

            if response.data:
                user_info = response.data[0]
                name_parts = []
                if user_info.get("first_name"):
                    name_parts.append(user_info["first_name"])
                if user_info.get("last_name"):
                    name_parts.append(user_info["last_name"])

                name = " ".join(name_parts) if name_parts else ""

                if user_info.get("username"):
                    if name:
                        return f"{name} (@{user_info['username']})"
                    else:
                        return f"@{user_info['username']}"
                elif name:
                    return name
                else:
                    return f"ID {user_id}"
            else:
                return f"ID {user_id}"

        except Exception as e:
            logger.error(f"Ошибка получения информации о пользователе {user_id}: {e}")
            return f"ID {user_id}"

    def _truncate_message(self, text: str, max_lines: int = 6) -> str:
        """Сокращает длинные сообщения"""
        if not text:
            return ""

        lines = text.split("\n")

        if len(lines) <= max_lines:
            return text

        # Берем первые 3 и последние 3 строки
        first_lines = lines[:3]
        last_lines = lines[-3:]

        truncated = "\n".join(first_lines) + "\n...\n" + "\n".join(last_lines)
        return truncated

    async def end_admin_conversation(self, admin_id: int) -> bool:
        """Завершает текущий диалог админа"""
        try:
            await self.supabase.end_admin_conversations(admin_id)
            logger.info(f"Завершен диалог админа {admin_id}")
            return True

        except Exception as e:
            logger.error(f"Ошибка завершения диалога админа {admin_id}: {e}")
            return False

    async def is_user_in_admin_chat(self, user_id: int) -> Optional[Dict[str, Any]]:
        """Проверяет, ведется ли диалог с пользователем"""
        try:
            logger.debug(f"🔍 Проверяем диалог с пользователем {user_id}")

            conversation = await self.supabase.get_user_admin_conversation(user_id)

            if conversation:
                logger.debug(
                    f"✅ Найден активный диалог: админ {conversation['admin_id']}, ID: {conversation['id']}"
                )
            else:
                logger.debug(f"❌ Активный диалог не найден для пользователя {user_id}")

            return conversation

        except Exception as e:
            logger.error(f"❌ Ошибка проверки диалога пользователя {user_id}: {e}")
            return None

    async def get_admin_active_conversation(
        self, admin_id: int
    ) -> Optional[Dict[str, Any]]:
        """Получает активный диалог админа"""
        try:
            return await self.supabase.get_admin_active_conversation(admin_id)

        except Exception as e:
            logger.error(f"Ошибка получения активного диалога админа {admin_id}: {e}")
            return None

    async def forward_message_to_admin(
        self, message: Message, conversation: Dict[str, Any]
    ):
        """Пересылает сообщение пользователя админу"""
        from .message_sender import send_message_by_human

        admin_id = conversation["admin_id"]
        user_id = message.from_user.id

        logger.info(f"📤 Пересылаем сообщение от {user_id} админу {admin_id}")

        # Форматируем сообщение для админа
        user_info = self._format_user_info(message.from_user)

        # Время с начала диалога
        try:
            start_time = datetime.fromisoformat(
                conversation["started_at"].replace("Z", "+00:00")
            )
            duration = datetime.now(start_time.tzinfo) - start_time
            minutes = int(duration.total_seconds() / 60)
        except Exception as e:
            logger.error(f"Ошибка расчета времени диалога: {e}")
            minutes = 0

        # ✅ ИСПРАВЛЕНИЕ: Убираем символы, которые могут вызвать ошибки парсинга
        safe_user_info = self._escape_markdown(user_info)
        safe_message_text = self._escape_markdown(message.text or "")

        header = f"👤 {safe_user_info} | ⏱️ {minutes} мин"
        separator = "━" * 20

        full_message = f"{header}\n{separator}\n{safe_message_text}"

        try:
            logger.info(f"📨 Отправляем сообщение админу {admin_id}")

            # ✅ ИСПРАВЛЕНИЕ: Убираем parse_mode='Markdown' чтобы избежать ошибок парсинга
            await send_message_by_human(admin_id, full_message)

            logger.info(f"✅ Сообщение успешно отправлено админу {admin_id}")

            # Добавляем кнопки управления
            await self._send_admin_controls(admin_id, user_id)

        except Exception as e:
            logger.error(f"❌ Ошибка пересылки сообщения админу {admin_id}: {e}")

            # ✅ ДОБАВЛЯЕМ: Fallback отправка без форматирования
            try:
                simple_message = f"Сообщение от пользователя {user_id}:\n{message.text}"
                await send_message_by_human(admin_id, simple_message)
                logger.info(f"✅ Простое сообщение отправлено админу {admin_id}")
            except Exception as e2:
                logger.error(f"❌ Даже простое сообщение не отправилось: {e2}")
                raise

    def _escape_markdown(self, text: str) -> str:
        """Экранирует специальные символы Markdown"""
        if not text:
            return ""

        # Символы, которые нужно экранировать в Markdown
        markdown_chars = [
            "*",
            "_",
            "`",
            "[",
            "]",
            "(",
            ")",
            "~",
            ">",
            "#",
            "+",
            "-",
            "=",
            "|",
            "{",
            "}",
            ".",
            "!",
        ]

        escaped_text = text
        for char in markdown_chars:
            escaped_text = escaped_text.replace(char, f"\\{char}")

        return escaped_text

    async def forward_message_to_user(
        self, message: Message, conversation: Dict[str, Any]
    ):
        """Пересылает сообщение админа пользователю"""
        from .message_sender import send_message_by_human

        supabase_client = get_global_var("supabase_client")

        user_id = conversation["user_id"]

        try:
            # Отправляем сообщение как от бота
            await send_message_by_human(user_id, message.text)

            # Сохраняем в БД как сообщение ассистента
            session_info = await supabase_client.get_active_session(user_id)
            if session_info:
                await supabase_client.add_message(
                    session_id=session_info["id"],
                    role="assistant",
                    content=message.text,
                    message_type="text",
                    metadata={"from_admin": message.from_user.id},
                )

        except Exception as e:
            logger.error(f"Ошибка пересылки сообщения пользователю {user_id}: {e}")

    async def _send_admin_controls(self, admin_id: int, user_id: int):
        """Отправляет кнопки управления диалогом"""
        from aiogram.types import InlineKeyboardButton, InlineKeyboardMarkup

        from .message_sender import send_message_by_human

        keyboard = InlineKeyboardMarkup(
            inline_keyboard=[
                [
                    InlineKeyboardButton(
                        text="📋 История", callback_data=f"admin_history_{user_id}"
                    ),
                    InlineKeyboardButton(
                        text="✅ Завершить", callback_data=f"admin_end_{user_id}"
                    ),
                ]
            ]
        )

        try:
            await send_message_by_human(
                admin_id, "🎛️ Управление диалогом:", reply_markup=keyboard
            )
        except Exception as e:
            logger.error(f"Ошибка отправки кнопок управления: {e}")

    def _format_user_info(self, user: User) -> str:
        """Форматирует информацию о пользователе"""
        name_parts = []
        if user.first_name:
            name_parts.append(user.first_name)
        if user.last_name:
            name_parts.append(user.last_name)

        name = " ".join(name_parts) if name_parts else "Без имени"

        if user.username:
            return f"{name} (@{user.username})"
        else:
            return f"{name} (ID: {user.id})"

    async def cleanup_expired_conversations(self):
        """Очищает просроченные диалоги"""
        try:
            ended_count = await self.supabase.end_expired_conversations()
            if ended_count > 0:
                logger.info(f"Очищено {ended_count} просроченных диалогов")
            return ended_count

        except Exception as e:
            logger.error(f"Ошибка очистки просроченных диалогов: {e}")
            return 0

    async def get_conversation_stats(self) -> Dict[str, int]:
        """Возвращает статистику диалогов"""
        try:
            # Здесь можно добавить запросы к БД для получения статистики
            # Пока возвращаем заглушку
            return {
                "active_conversations": 0,
                "completed_today": 0,
                "total_admin_messages": 0,
            }
        except Exception as e:
            logger.error(f"Ошибка получения статистики диалогов: {e}")
            return {}

    async def get_active_conversations(self) -> List[Dict[str, Any]]:
        """Получает все активные диалоги админов"""
        try:
            logger.info("🔍 Ищем активные диалоги админов...")

            # Получаем все активные диалоги
            query = (
                self.supabase.client.table("admin_user_conversations")
                .select("id", "admin_id", "user_id", "started_at", "auto_end_at")
                .eq("status", "active")
            )

            # Добавляем фильтр по bot_id если он указан
            if self.supabase.bot_id:
                query = query.eq("bot_id", self.supabase.bot_id)

            response = query.order("started_at", desc=True).execute()

            logger.info(f"📊 Найдено {len(response.data)} активных диалогов в БД")

            conversations = []
            for conv in response.data:
                # Получаем информацию о пользователе
                try:
                    user_query = (
                        self.supabase.client.table("sales_users")
                        .select("first_name", "last_name", "username")
                        .eq("telegram_id", conv["user_id"])
                    )

                    # Добавляем фильтр по bot_id если он указан
                    if self.supabase.bot_id:
                        user_query = user_query.eq("bot_id", self.supabase.bot_id)

                    user_response = user_query.execute()

                    user_info = user_response.data[0] if user_response.data else {}
                except Exception as e:
                    logger.error(
                        f"Ошибка получения данных пользователя {conv['user_id']}: {e}"
                    )
                    user_info = {}

                # Получаем информацию об админе
                try:
                    admin_query = (
                        self.supabase.client.table("sales_admins")
                        .select("first_name", "last_name", "username")
                        .eq("telegram_id", conv["admin_id"])
                    )

                    # Добавляем фильтр по bot_id если он указан
                    if self.supabase.bot_id:
                        admin_query = admin_query.eq("bot_id", self.supabase.bot_id)

                    admin_response = admin_query.execute()

                    admin_info = admin_response.data[0] if admin_response.data else {}
                except Exception as e:
                    logger.error(
                        f"Ошибка получения данных админа {conv['admin_id']}: {e}"
                    )
                    admin_info = {}

                conv["user_info"] = user_info
                conv["admin_info"] = admin_info
                conversations.append(conv)

            logger.info(
                f"✅ Получено {len(conversations)} активных диалогов с дополнительной информацией"
            )
            return conversations

        except Exception as e:
            logger.error(f"❌ Ошибка получения активных диалогов: {e}")
            return []

    def format_active_conversations(self, conversations: List[Dict[str, Any]]) -> str:
        """Форматирует список активных диалогов - ИСПРАВЛЕН РАСЧЕТ ВРЕМЕНИ АВТОЗАВЕРШЕНИЯ"""
        if not conversations:
            return "💬 Нет активных диалогов"

        lines = ["💬 АКТИВНЫЕ ДИАЛОГИ:", ""]

        for i, conv in enumerate(conversations, 1):
            # Информация о пользователе
            user_info = conv.get("user_info", {})
            user_name = []
            if user_info.get("first_name"):
                user_name.append(user_info["first_name"])
            if user_info.get("last_name"):
                user_name.append(user_info["last_name"])

            user_display = " ".join(user_name) if user_name else f"ID {conv['user_id']}"
            if user_info.get("username"):
                user_display += f" (@{user_info['username']})"

            # Информация об админе
            admin_info = conv.get("admin_info", {})
            admin_name = []
            if admin_info.get("first_name"):
                admin_name.append(admin_info["first_name"])
            if admin_info.get("last_name"):
                admin_name.append(admin_info["last_name"])

            admin_display = (
                " ".join(admin_name) if admin_name else f"ID {conv['admin_id']}"
            )

            # 🔧 ИСПРАВЛЕНИЕ: Правильный расчет времени с учетом timezone
            try:
                started_at_str = conv["started_at"]
                logger.debug(f"🕐 Диалог {i}: started_at = '{started_at_str}'")

                # Парсим время начала с правильной обработкой timezone
                if started_at_str.endswith("Z"):
                    start_time = datetime.fromisoformat(
                        started_at_str.replace("Z", "+00:00")
                    )
                elif "+" in started_at_str or started_at_str.count(":") >= 3:
                    # Уже есть timezone info
                    start_time = datetime.fromisoformat(started_at_str)
                else:
                    # Если нет timezone info, считаем что это UTC
                    naive_time = datetime.fromisoformat(started_at_str)
                    start_time = naive_time.replace(tzinfo=timezone.utc)

                logger.debug(f"✅ Парсед start_time: {start_time}")

                # Получаем текущее время в UTC
                now_utc = datetime.now(timezone.utc)
                logger.debug(f"🕐 now_utc: {now_utc}")

                # Приводим start_time к UTC если нужно
                if start_time.tzinfo != timezone.utc:
                    start_time_utc = start_time.astimezone(timezone.utc)
                else:
                    start_time_utc = start_time

                # Длительность диалога
                duration = now_utc - start_time_utc
                minutes = max(0, int(duration.total_seconds() / 60))
                logger.debug(f"⏱️ Длительность: {minutes} минут")

            except Exception as e:
                logger.error(f"❌ Ошибка расчета времени диалога {i}: {e}")
                logger.error(f"   started_at_str: '{started_at_str}'")
                minutes = 0

            # 🔧 ИСПРАВЛЕНИЕ: Время до автозавершения с правильной обработкой timezone
            try:
                auto_end_str = conv["auto_end_at"]
                logger.debug(f"🕐 Диалог {i}: auto_end_at = '{auto_end_str}'")

                # Парсим время автозавершения с правильной обработкой timezone
                if auto_end_str.endswith("Z"):
                    auto_end = datetime.fromisoformat(
                        auto_end_str.replace("Z", "+00:00")
                    )
                elif "+" in auto_end_str or auto_end_str.count(":") >= 3:
                    # Уже есть timezone info
                    auto_end = datetime.fromisoformat(auto_end_str)
                else:
                    # Если нет timezone info, считаем что это UTC
                    naive_time = datetime.fromisoformat(auto_end_str)
                    auto_end = naive_time.replace(tzinfo=timezone.utc)

                logger.debug(f"✅ Парсед auto_end: {auto_end}")

                # Получаем текущее время в UTC
                now_utc = datetime.now(timezone.utc)
                logger.debug(f"🕐 now_utc для auto_end: {now_utc}")

                # Приводим auto_end к UTC если нужно
                if auto_end.tzinfo != timezone.utc:
                    auto_end_utc = auto_end.astimezone(timezone.utc)
                else:
                    auto_end_utc = auto_end

                # Оставшееся время
                remaining = auto_end_utc - now_utc
                remaining_minutes = max(0, int(remaining.total_seconds() / 60))
                logger.debug(f"⏰ Remaining: {remaining_minutes} минут")

                # 🔧 ДОПОЛНИТЕЛЬНАЯ ПРОВЕРКА: вычисляем планируемую длительность
                if start_time.tzinfo != timezone.utc:
                    start_time_utc = start_time.astimezone(timezone.utc)
                else:
                    start_time_utc = start_time

                planned_duration = auto_end_utc - start_time_utc
                planned_minutes = int(planned_duration.total_seconds() / 60)
                logger.debug(f"📏 Планируемая длительность: {planned_minutes} минут")

                # Проверяем корректность
                expected_timeout = self.admin_session_timeout_minutes

                logger.debug(f"🕐 expected_timeout: {expected_timeout}")

                if (
                    abs(planned_minutes - expected_timeout) > 2
                ):  # допускаем погрешность в 2 минуты
                    logger.warning(
                        f"⚠️ Диалог {i}: планируемая длительность {planned_minutes} мин не соответствует конфигу {expected_timeout} мин"
                    )

            except Exception as e:
                logger.error(
                    f"❌ Ошибка расчета времени автозавершения диалога {i}: {e}"
                )
                logger.error(f"   auto_end_str: '{auto_end_str}'")
                remaining_minutes = 0

            lines.append(f"{i}. 👤 {user_display}")
            lines.append(f"   👑 Админ: {admin_display}")
            lines.append(f"   ⏱️ Длительность: {minutes} мин")
            lines.append(f"   ⏰ Автозавершение через: {remaining_minutes} мин")
            lines.append(f"   🎛️ /чат {conv['user_id']}")
            lines.append("")

        return "\n".join(lines)

    async def route_admin_message(self, message: Message, state: FSMContext) -> bool:
        """
        Маршрутизирует сообщение админа
        Возвращает True если сообщение обработано как админское
        """
        admin_id = message.from_user.id

        # Проверяем админские команды
        if message.text and message.text.startswith("/"):
            return False  # Команды обрабатываются отдельно

        # Проверяем, ведется ли диалог с пользователем
        conversation = await self.get_admin_active_conversation(admin_id)

        if conversation:
            # Пересылаем сообщение пользователю
            await self.forward_message_to_user(message, conversation)
            return True

        return False



"""
Утилита для тестирования системы администрирования бота
"""

import asyncio
import logging
import sys

from ..analytics.analytics_manager import AnalyticsManager
from ..config import Config
from ..core.conversation_manager import ConversationManager
from ..integrations.supabase_client import SupabaseClient
from .admin_manager import AdminManager
from .timeout_checker import setup_bot_environment

logger = logging.getLogger(__name__)


async def test_admin_system(bot_name: str = "growthmed-october-24") -> bool:
    """
    Тестирует систему администрирования бота

    Args:
        bot_name: Имя бота для тестирования

    Returns:
        bool: True если все тесты пройдены, False если найдены проблемы
    """
    logger.info(f"🚀 Тестирование системы администрирования: {bot_name}")
    logger.info(f"🤖 Bot ID будет автоопределен как: {bot_name}\n")

    # Настраиваем окружение для бота (автоматически устанавливает BOT_ID)
    config_dir = setup_bot_environment(bot_name)
    if not config_dir:
        return False

    # Инициализируем конфигурацию
    config = Config()
    logger.info("📋 Конфигурация:")
    logger.info(f"   BOT_ID: {config.BOT_ID}")
    logger.info(
        f"   ADMIN_SESSION_TIMEOUT_MINUTES: {config.ADMIN_SESSION_TIMEOUT_MINUTES}"
    )
    logger.info(f"   PROMT_FILES_DIR: {config.PROMT_FILES_DIR}")
    logger.info(f"   Найдено промпт-файлов: {len(config.PROMPT_FILES)}")
    logger.info("")

    # Проверяем админов
    if not config.ADMIN_TELEGRAM_IDS:
        logger.warning("⚠️ Админы не настроены (ADMIN_TELEGRAM_IDS пуст)")
        return False

    logger.info(f"👑 Админы: {config.ADMIN_TELEGRAM_IDS}")
    logger.info("")

    # Проверяем подключение к БД
    try:
        supabase_client = SupabaseClient(config.SUPABASE_URL, config.SUPABASE_KEY)
        await supabase_client.initialize()
        logger.info("✅ Подключение к Supabase успешно")

        # Проверяем таблицы
        tables = [
            "sales_admins",
            "admin_user_conversations",
            "session_events",
            "sales_chat_sessions",
            "sales_messages",
        ]

        logger.info("\n📊 Проверка таблиц:")
        for table in tables:
            try:
                supabase_client.client.table(table).select("*").limit(1).execute()
                logger.info(f"   ✅ {table}")
            except Exception as e:
                logger.error(f"   ❌ {table}: {e}")
                return False

        # Проверяем AdminManager
        admin_manager = AdminManager(config, supabase_client)
        logger.info(
            f"\n👑 AdminManager инициализирован ({len(admin_manager.admin_ids)} админов)"
        )

        # Проверяем ConversationManager
        conversation_manager = ConversationManager(supabase_client, admin_manager)
        logger.info("✅ ConversationManager инициализирован")

        # Проверяем AnalyticsManager
        analytics_manager = AnalyticsManager(supabase_client)

        # Тестируем получение статистики
        await analytics_manager.get_funnel_stats(1)
        logger.info("✅ AnalyticsManager работает")

        # Проверяем активные диалоги
        conversations = await conversation_manager.get_active_conversations()
        logger.info(f"\n💬 Активные диалоги: {len(conversations)}")

        if conversations:
            for conv in conversations:
                logger.info(
                    f"   • Диалог {conv['id']}: админ {conv['admin_id']} с пользователем {conv['user_id']}"
                )
        else:
            logger.info("   Нет активных диалогов")
            logger.info("   💡 Создайте диалог командой /чат USER_ID для тестирования")

        # Проверяем форматирование диалогов
        if conversations:
            formatted = conversation_manager.format_active_conversations(conversations)
            logger.info("\n📝 Форматирование диалогов:")
            logger.info(formatted)

        logger.info("\n✅ Админская система готова к работе")
        return True

    except Exception as e:
        logger.error(f"❌ Ошибка тестирования: {e}")
        logger.exception("Стек ошибки:")
        return False


def main():
    """Точка входа для запуска из командной строки"""
    # Настройка логирования
    logging.basicConfig(level=logging.INFO, format="%(message)s")

    logger.info("🔍 Утилита тестирования админской системы")
    logger.info("Использование:")
    logger.info("  python -m smart_bot_factory.admin_tester [bot_name]")
    logger.info("  python -m smart_bot_factory.admin_tester growthmed-october-24")
    logger.info("")

    if len(sys.argv) > 1 and sys.argv[1] in ["-h", "--help", "help"]:
        return

    # Определяем какого бота тестировать
    bot_name = "growthmed-october-24"  # по умолчанию
    if len(sys.argv) > 1:
        bot_name = sys.argv[1]

    try:
        success = asyncio.run(test_admin_system(bot_name))
        if not success:
            sys.exit(1)
    except KeyboardInterrupt:
        logger.info("\n⏹️ Прервано пользователем")
    except Exception as e:
        logger.error(f"\n💥 Критическая ошибка: {e}")
        logger.exception("Стек ошибки:")
        sys.exit(1)


if __name__ == "__main__":
    main()



# Обновленный prompt_loader.py с поддержкой финальных инструкций

import logging
from pathlib import Path
from typing import Dict

import aiofiles

logger = logging.getLogger(__name__)


class PromptLoader:
    """Класс для загрузки промптов из локального каталога"""

    def __init__(self, prompts_dir: str):
        self.prompts_dir = Path(prompts_dir)
        self.welcome_file = self.prompts_dir / "welcome_message.txt"
        self.help_file = self.prompts_dir / "help_message.txt"
        self.final_instructions_file = self.prompts_dir / "final_instructions.txt"
        
        # Информация об инструментах для добавления в финальные инструкции
        self._tools_description: str = ""

        # Автоматически находим все .txt файлы промптов (кроме специальных)
        all_txt_files = list(self.prompts_dir.glob("*.txt"))
        special_files = {
            "welcome_message.txt",
            "help_message.txt",
            "final_instructions.txt",
        }
        self.prompt_files = [
            f.name for f in all_txt_files if f.name not in special_files
        ]

        logger.info(f"Инициализирован загрузчик промптов: {self.prompts_dir}")
        logger.info(f"Найдено файлов промптов: {len(self.prompt_files)}")
        logger.info(f"Файлы промптов: {self.prompt_files}")

    async def load_system_prompt(self) -> str:
        """
        Загружает и объединяет все файлы промптов в один системный промпт

        Returns:
            Объединенный системный промпт с инструкциями по JSON
        """
        try:
            prompt_parts = []

            for filename in self.prompt_files:
                logger.debug(f"Загружаем промпт из {filename}")
                content = await self._load_file(filename)

                if content:
                    # Добавляем заголовок секции
                    section_name = self._get_section_name(filename)
                    prompt_parts.append(f"\n### {section_name} ###\n")
                    prompt_parts.append(content.strip())
                    prompt_parts.append("\n")
                else:
                    logger.warning(f"Файл {filename} пуст")

            if not prompt_parts:
                error_msg = "Не удалось загрузить ни одного промпт файла"
                logger.error(error_msg)
                raise ValueError(error_msg)

            # Добавляем инструкции по JSON метаданным
            json_instructions = self._get_json_instructions()
            prompt_parts.append("\n")
            prompt_parts.append(json_instructions)

            # Объединяем все части
            full_prompt = "".join(prompt_parts).strip()

            logger.info(
                f"Системный промпт загружен успешно ({len(full_prompt)} символов)"
            )
            return full_prompt

        except Exception as e:
            logger.error(f"Ошибка при загрузке системного промпта: {e}")
            raise

    def set_tools_description(self, tools_description: str):
        """
        Устанавливает описание инструментов для добавления в финальные инструкции
        
        Args:
            tools_description: Текст с описанием инструментов
        """
        self._tools_description = tools_description
        logger.info(f"📝 Описание инструментов обновлено ({len(tools_description)} символов)")
        
        # Логируем содержимое описания инструментов
        if tools_description:
            # Подсчитываем количество инструментов по паттерну "1. **название**"
            import re
            tool_pattern = r'\d+\.\s+\*\*[^*]+\*\*'
            tool_matches = re.findall(tool_pattern, tools_description)
            tool_count = len(tool_matches)
            logger.info(f"📋 Описание инструментов для промпта содержит {tool_count} инструмент(ов)")
            
            # Логируем первые строки для проверки
            lines = tools_description.split('\n')
            non_empty_lines = [line for line in lines if line.strip()]
            preview_lines = non_empty_lines[:10]  # Первые 10 непустых строк
            
            logger.info("📋 Превью описания инструментов (первые строки):")
            for line in preview_lines:
                logger.info(f"   {line}")
            
            if len(non_empty_lines) > 10:
                logger.info(f"   ... (еще {len(non_empty_lines) - 10} строк)")
            
            # Полное содержимое в debug режиме
            logger.debug(f"📋 Полное содержимое описания инструментов:\n{tools_description}")

    async def load_final_instructions(self) -> str:
        """
        Загружает финальные инструкции из final_instructions.txt
        Автоматически добавляет информацию об инструментах если она установлена

        Returns:
            Финальные инструкции или пустая строка если файла нет
        """
        try:
            logger.debug(
                f"Загружаем финальные инструкции из {self.final_instructions_file.name}"
            )

            content = ""
            
            if self.final_instructions_file.exists():
                async with aiofiles.open(
                    self.final_instructions_file, "r", encoding="utf-8"
                ) as f:
                    content = await f.read()

            if not content.strip():
                if not self.final_instructions_file.exists():
                    logger.debug(
                        f"Файл {self.final_instructions_file.name} не найден - пропускаем"
                    )
                else:
                    logger.debug(
                        f"Файл {self.final_instructions_file.name} пуст - пропускаем"
                    )
                
                # Если есть описание инструментов, возвращаем только его
                if self._tools_description:
                    logger.info(f"Финальные инструкции: только описание инструментов ({len(self._tools_description)} символов)")
                    logger.debug(f"Содержимое описания инструментов (первые 500 символов):\n{self._tools_description[:500]}...")
                    return self._tools_description.strip()
                
                return ""

            # Добавляем описание инструментов в конец финальных инструкций
            if self._tools_description:
                # Проверяем, не добавлено ли уже описание инструментов
                if "### ДОСТУПНЫЕ ИНСТРУМЕНТЫ ###" not in content:
                    original_length = len(content)
                    content = content.strip() + "\n\n" + self._tools_description
                    added_length = len(content) - original_length
                    logger.info(f"✅ Описание инструментов добавлено в финальные инструкции")
                    logger.info(f"   Размер исходных инструкций: {original_length} символов")
                    logger.info(f"   Добавлено символов: {added_length}")
                    logger.info(f"   Итоговый размер: {len(content)} символов")
                    logger.debug(f"   Добавленный текст (первые 500 символов):\n{self._tools_description[:500]}...")
                else:
                    logger.debug("Описание инструментов уже присутствует в финальных инструкциях")

            logger.info(f"Финальные инструкции загружены ({len(content)} символов)")
            return content.strip()

        except Exception as e:
            logger.error(f"Ошибка при загрузке финальных инструкций: {e}")
            # Не прерываем работу - финальные инструкции опциональны
            # Но возвращаем описание инструментов если оно есть
            if self._tools_description:
                return self._tools_description.strip()
            return ""

    def _get_json_instructions(self) -> str:
        """Возвращает инструкции по JSON метаданным для ИИ"""
        return """
### КРИТИЧЕСКИ ВАЖНЫЕ ИНСТРУКЦИИ ПО JSON МЕТАДАННЫМ ###

В КОНЦЕ КАЖДОГО своего ответа ОБЯЗАТЕЛЬНО добавляй служебную информацию в JSON формате:

{
  "этап": "introduction|consult|offer|contacts",
  "качество": 1-10,
  "ссылка": 1,
  "события": [
    {
      "тип": "телефон|консультация|покупка|отказ",
      "инфо": "детали события"
    }
  ],
  "файлы": ["file1.pdf", "file2.mp4"],
  "каталоги": ["каталог1", "каталог2"]
}

ОПИСАНИЕ ЭТАПОВ:
- introduction: знакомство, сбор базовой информации о клиенте
- consult: консультирование, ответы на вопросы о конференции
- offer: предложение тарифов, обработка возражений
- contacts: получение контактов, завершение сделки

СИСТЕМА ОЦЕНКИ КАЧЕСТВА (1-10):
1-3: низкий интерес, много возражений, скептически настроен
4-6: средний интерес, есть вопросы, обдумывает
7-8: высокий интерес, готов к покупке, активно интересуется
9-10: горячий лид, предоставил контакты или готов к действию


СОБЫТИЯ - добавляй ТОЛЬКО когда происходит что-то из этого:
- "телефон": пользователь предоставил номер телефона
- "консультация": пользователь просит живую консультацию по телефону
- "покупка": пользователь готов купить/записаться на конференцию
- "отказ": пользователь явно отказывается участвовать

ВАЖНО:
- Добавляй файлы и катологи только в том случае, если они относятся к этому этапу. (Прописаны в самом этапе) Если не относятся - отсылай пустой массив []

ПРИМЕРЫ ПРАВИЛЬНОГО ИСПОЛЬЗОВАНИЯ:

Пример 1 - обычный диалог (без ссылки):
"Расскажу подробнее о конференции GrowthMED. Она пройдет 24-25 октября..."

{
  "этап": "consult",
  "качество": 6,
  "события": [],
  "файлы": [],
  "каталоги": []
}

Пример 2 - получен телефон (без ссылки):
"Отлично! Записал ваш номер. Мы перезвоним в течение 10 минут!"

{
  "этап": "contacts",
  "качество": 9,
  "события": [
    {
      "тип": "телефон",
      "инфо": "Иван Петров +79219603144"
    }
  ],
  "файлы": [],
  "каталоги": []
}

Пример 3 - отправка презентации (без ссылки):
"Отправляю вам презентацию о нашей компании и прайс-лист с актуальными ценами."

{
  "этап": "offer",
  "качество": 7,
  "события": [
    {
      "тип": "консультация",
      "инфо": "Запросил материалы"
    }
  ],
  "файлы": ["презентация.pdf", "прайс.pdf"],
  "каталоги": []
}

Пример 4 - отправка файлов из каталога (без ссылки):
"В каталоге 'примеры_работ' вы можете посмотреть наши последние проекты."

{
  "этап": "presentation",
  "качество": 8,
  "события": [],
  "файлы": [],
  "каталоги": ["примеры_работ"]
}

Пример 5 - комбинированная отправка (без ссылки):
"Отправляю вам коммерческое предложение и примеры похожих проектов из нашего портфолио."

{
  "этап": "offer",
  "качество": 9,
  "события": [
    {
      "тип": "предложение",
      "инфо": "Отправлено КП"
    }
  ],
  "файлы": ["коммерческое_предложение.pdf"],
  "каталоги": ["портфолио_2023"]
}


ТРЕБОВАНИЯ К JSON:
- JSON должен быть валидным и находиться в самом конце ответа
- Всегда используй кавычки для строк
- Массив "события" может быть пустым []
- Если событий нет - не добавляй их в массив
- Качество должно быть числом от 1 до 10


ПОМНИ: Этот JSON критически важен для работы системы администрирования и аналитики!
"""

    async def load_welcome_message(self) -> str:
        """
        Загружает приветственное сообщение из welcome_message.txt

        Returns:
            Текст приветственного сообщения
        """
        try:
            logger.debug(
                f"Загружаем приветственное сообщение из {self.welcome_file.name}"
            )

            if not self.welcome_file.exists():
                error_msg = f"Файл приветствия не найден: {self.welcome_file}"
                logger.error(error_msg)
                raise FileNotFoundError(error_msg)

            async with aiofiles.open(self.welcome_file, "r", encoding="utf-8") as f:
                content = await f.read()

            if not content.strip():
                error_msg = f"Файл приветствия пуст: {self.welcome_file}"
                logger.error(error_msg)
                raise ValueError(error_msg)

            logger.info(f"Приветственное сообщение загружено ({len(content)} символов)")
            return content.strip()

        except Exception as e:
            logger.error(f"Ошибка при загрузке приветственного сообщения: {e}")
            raise

    async def load_help_message(self) -> str:
        """
        Загружает справочное сообщение из help_message.txt

        Returns:
            Текст справочного сообщения
        """
        try:
            logger.debug(f"Загружаем справочное сообщение из {self.help_file.name}")

            if self.help_file.exists():
                async with aiofiles.open(self.help_file, "r", encoding="utf-8") as f:
                    content = await f.read()

                if content.strip():
                    logger.info(
                        f"Справочное сообщение загружено ({len(content)} символов)"
                    )
                    return content.strip()

            # Fallback если файл не найден или пуст
            logger.warning(
                "Файл help_message.txt не найден или пуст, используем дефолтную справку"
            )
            return "🤖 **Ваш помощник готов к работе!**\n\n**Команды:**\n/start - Начать диалог\n/help - Показать справку\n/status - Проверить статус"

        except Exception as e:
            logger.error(f"Ошибка при загрузке справочного сообщения: {e}")
            # Возвращаем простую справку в случае ошибки
            return "🤖 Ваш помощник готов к работе! Напишите /start для начала диалога."

    async def _load_file(self, filename: str) -> str:
        """Загружает содержимое файла из каталога промптов"""
        file_path = self.prompts_dir / filename

        try:
            if not file_path.exists():
                error_msg = f"Файл промпта не найден: {file_path}"
                logger.error(error_msg)
                raise FileNotFoundError(error_msg)

            async with aiofiles.open(file_path, "r", encoding="utf-8") as f:
                content = await f.read()

            if not content.strip():
                logger.warning(f"Файл {filename} пуст")
                return ""

            logger.debug(f"Загружен файл {filename} ({len(content)} символов)")
            return content

        except Exception as e:
            logger.error(f"Ошибка чтения файла {file_path}: {e}")
            raise

    def _get_section_name(self, filename: str) -> str:
        """Получает название секции по имени файла"""
        name_mapping = {
            "system_prompt.txt": "СИСТЕМНЫЙ ПРОМПТ",
            "sales_context.txt": "КОНТЕКСТ ПРОДАЖ",
            "product_info.txt": "ИНФОРМАЦИЯ О ПРОДУКТЕ",
            "objection_handling.txt": "ОБРАБОТКА ВОЗРАЖЕНИЙ",
            "1sales_context.txt": "КОНТЕКСТ ПРОДАЖ",
            "2product_info.txt": "ИНФОРМАЦИЯ О ПРОДУКТЕ",
            "3objection_handling.txt": "ОБРАБОТКА ВОЗРАЖЕНИЙ",
            "final_instructions.txt": "ФИНАЛЬНЫЕ ИНСТРУКЦИИ",  # 🆕
        }

        return name_mapping.get(filename, filename.replace(".txt", "").upper())

    async def reload_prompts(self) -> str:
        """Перезагружает промпты (для обновления без перезапуска бота)"""
        logger.info("Перезагрузка промптов...")
        return await self.load_system_prompt()

    async def validate_prompts(self) -> Dict[str, bool]:
        """Проверяет доступность всех файлов промптов и приветственного сообщения"""
        results = {}

        # Проверяем файлы промптов
        for filename in self.prompt_files:
            file_path = self.prompts_dir / filename
            try:
                if file_path.exists():
                    async with aiofiles.open(file_path, "r", encoding="utf-8") as f:
                        content = await f.read()
                    results[filename] = bool(
                        content.strip() and len(content.strip()) > 10
                    )
                else:
                    results[filename] = False
            except Exception:
                results[filename] = False

        # Проверяем файл приветственного сообщения
        try:
            if self.welcome_file.exists():
                async with aiofiles.open(self.welcome_file, "r", encoding="utf-8") as f:
                    content = await f.read()
                results["welcome_message.txt"] = bool(
                    content.strip() and len(content.strip()) > 5
                )
            else:
                results["welcome_message.txt"] = False
        except Exception:
            results["welcome_message.txt"] = False

        # Проверяем файл справки (опционально)
        try:
            if self.help_file.exists():
                async with aiofiles.open(self.help_file, "r", encoding="utf-8") as f:
                    content = await f.read()
                results["help_message.txt"] = bool(
                    content.strip() and len(content.strip()) > 5
                )
            else:
                results["help_message.txt"] = False  # Не критично
        except Exception:
            results["help_message.txt"] = False

        # 🆕 Проверяем финальные инструкции (опционально)
        try:
            if self.final_instructions_file.exists():
                async with aiofiles.open(
                    self.final_instructions_file, "r", encoding="utf-8"
                ) as f:
                    content = await f.read()
                results["final_instructions.txt"] = bool(
                    content.strip() and len(content.strip()) > 5
                )
            else:
                results["final_instructions.txt"] = (
                    False  # Не критично - опциональный файл
                )
        except Exception:
            results["final_instructions.txt"] = False

        return results

    def get_prompt_info(self) -> Dict[str, any]:
        """Возвращает информацию о конфигурации промптов"""
        return {
            "prompts_dir": str(self.prompts_dir),
            "prompt_files": self.prompt_files,
            "welcome_file": "welcome_message.txt",
            "help_file": "help_message.txt",
            "final_instructions_file": "final_instructions.txt",  # 🆕
            "total_files": len(self.prompt_files) + 1,  # +1 для welcome message
            "json_instructions_included": True,
        }

    async def test_json_parsing(self, test_response: str) -> Dict[str, any]:
        """Тестирует парсинг JSON из ответа ИИ (для отладки)"""
        import json
        import re

        try:
            # Используем тот же алгоритм что и в main.py
            json_pattern = r'\{[^{}]*"этап"[^{}]*\}$'
            match = re.search(json_pattern, test_response.strip())

            if match:
                json_str = match.group(0)
                response_text = test_response[: match.start()].strip()

                try:
                    metadata = json.loads(json_str)
                    return {
                        "success": True,
                        "response_text": response_text,
                        "metadata": metadata,
                        "json_str": json_str,
                    }
                except json.JSONDecodeError as e:
                    return {
                        "success": False,
                        "error": f"JSON decode error: {e}",
                        "json_str": json_str,
                    }
            else:
                return {
                    "success": False,
                    "error": "JSON pattern not found",
                    "response_text": test_response,
                }

        except Exception as e:
            return {"success": False, "error": f"Parse error: {e}"}



"""
Admin модули smart_bot_factory
"""

from .admin_events import setup_admin_events_handlers
from .admin_logic import setup_admin_handlers
from .admin_manager import AdminManager
from .admin_tester import test_admin_system
from .timeout_checker import check_timeouts, setup_bot_environment

__all__ = [
    "setup_admin_handlers",
    "setup_admin_events_handlers",
    "AdminManager",
    "test_admin_system",
    "check_timeouts",
    "setup_bot_environment",
]



import asyncio
import json
import logging
from typing import Any, Dict, List, Optional

from langchain_openai import ChatOpenAI
from langchain.messages import AIMessage, HumanMessage, SystemMessage
from langchain_core.output_parsers import StrOutputParser

from langchain.agents import create_agent

from openai import AsyncOpenAI

logger = logging.getLogger(__name__)


class LangChainOpenAIClient:
    """Клиент для работы с OpenAI API через LangChain v1.0"""

    def __init__(
        self,
        api_key: str,
        model: str = "gpt-5",
        max_tokens: int = 1500,
        temperature: float = 0.7,
    ):
        self.api_key = api_key
        self.model = model
        self.max_tokens = max_tokens
        self.temperature = temperature
        
        self.client = AsyncOpenAI(api_key=api_key)

        # Список инструментов для ChatOpenAI
        self._tools: List[Any] = []

        # Инициализируем базовую LangChain модель
        self._base_chat_model = ChatOpenAI(
            model=model,
            api_key=api_key,
            max_tokens=max_tokens,
            temperature=temperature,
            reasoning_effort="minimal" if self._is_gpt5() else None,
            max_retries=3
        )

        self.chat_model = create_agent(model=self._base_chat_model, tools=self._tools)
        
        # Получаем лимиты для модели
        self.model_limits = self._get_model_limits()

        # Для диагностики пустых ответов
        self.last_completion_tokens = 0

        logger.info(
            f"LangChain OpenAI клиент инициализирован с моделью {model} (GPT-5: {self._is_gpt5()}, лимит: {self.model_limits['total_context']} токенов)"
        )
        
        # Логируем информацию об инструментах
        self._log_tools()
        
    @property
    def is_gpt5(self) -> bool:
        """Определяет, является ли модель GPT-5"""
        return self._is_gpt5()
        
    def _is_gpt5(self) -> bool:
        """Определяет, является ли модель GPT-5"""
        return "gpt-5" in self.model.lower()

    def _get_model_limits(self) -> Dict[str, int]:
        """Возвращает лимиты для конкретной модели"""
        model_limits = {
            # GPT-3.5
            "gpt-3.5-turbo": 4096,
            "gpt-3.5-turbo-16k": 16385,
            # GPT-4
            "gpt-4": 8192,
            "gpt-4-32k": 32768,
            "gpt-4-turbo": 128000,
            "gpt-4-turbo-preview": 128000,
            "gpt-4o": 128000,
            "gpt-4o-mini": 128000,
            # GPT-5
            "gpt-5-mini": 128000,
            "gpt-5": 200000,
        }

        # Получаем лимит для текущей модели или используем консервативное значение
        total_limit = model_limits.get(self.model, 8192)

        # Резервируем место для ответа и буфера
        completion_reserve = min(
            self.max_tokens * 2, total_limit // 4
        )  # Резервируем место для ответа
        buffer_reserve = 500  # Буфер для безопасности

        return {
            "total_context": total_limit,
            "max_input_tokens": total_limit - completion_reserve - buffer_reserve,
            "completion_reserve": completion_reserve,
        }

    def _convert_messages_to_langchain(
        self, messages: List[Dict[str, str]]
    ) -> List[Any]:
        """Конвертирует сообщения из формата dict в LangChain messages"""
        langchain_messages = []
        for msg in messages:
            role = msg.get("role", "user")
            content = msg.get("content", "")

            if role == "system":
                langchain_messages.append(SystemMessage(content=content))
            elif role == "user":
                langchain_messages.append(HumanMessage(content=content))
            elif role == "assistant":
                langchain_messages.append(AIMessage(content=content))
            else:
                # По умолчанию считаем user
                langchain_messages.append(HumanMessage(content=content))

        return langchain_messages

    def _convert_langchain_to_dict(self, message: AIMessage) -> Dict[str, Any]:
        """Конвертирует LangChain сообщение обратно в dict для логирования"""
        return {
            "content": message.content if hasattr(message, "content") else str(message),
            "role": "assistant",
        }

    def _get_completion_params(
        self, max_tokens: Optional[int] = None, temperature: Optional[float] = None
    ) -> Dict[str, Any]:
        """Получает параметры для запроса в зависимости от модели"""
        params = {}

        if self.is_gpt5:
            # Для GPT-5 используем специальные параметры
            if max_tokens:
                params["max_completion_tokens"] = max_tokens
            params["reasoning_effort"] = "minimal"
        else:
            # Для обычных моделей
            if max_tokens:
                params["max_tokens"] = max_tokens
            elif self.max_tokens:
                params["max_tokens"] = self.max_tokens

            if temperature is not None:
                params["temperature"] = temperature
            elif self.temperature is not None:
                params["temperature"] = self.temperature

        return params

    def _log_tools(self):
        """Логирует информацию о всех подключенных инструментах"""
        if not self._tools:
            logger.info("📋 Подключенные инструменты ChatOpenAI: нет инструментов")
            return
        
        logger.info(f"📋 Подключенные инструменты ChatOpenAI ({len(self._tools)} шт.):")
        for i, tool in enumerate(self._tools, 1):
            tool_name = getattr(tool, "name", "Unknown")
            tool_description = getattr(tool, "description", "Нет описания")
            tool_type = type(tool).__name__
            
            # Обрезаем описание если слишком длинное
            if len(tool_description) > 100:
                tool_description = tool_description[:97] + "..."
            
            logger.info(f"   {i}. {tool_name} ({tool_type})")
            logger.info(f"      Описание: {tool_description}")

    def _update_agent(self):
        """Обновляет агента с текущим списком инструментов"""
        self.chat_model = create_agent(model=self._base_chat_model, tools=self._tools)
        logger.info(f"🔄 Агент обновлен с {len(self._tools)} инструментами")

    def add_tool(self, tool: Any, update_agent: bool = True, log_after: bool = True):
        """
        Добавляет инструмент для ChatOpenAI

        Args:
            tool: Инструмент LangChain (например, StructuredTool, FunctionTool и т.д.)
            update_agent: Обновлять ли агента после добавления (по умолчанию True)
            log_after: Логировать ли список инструментов после добавления (по умолчанию True)
        """
        if tool not in self._tools:
            self._tools.append(tool)
            tool_name = getattr(tool, "name", str(tool))
            tool_description = getattr(tool, "description", "Нет описания")
            tool_type = type(tool).__name__
            
            logger.info(f"✅ Инструмент '{tool_name}' ({tool_type}) добавлен в ChatOpenAI")
            if tool_description and tool_description != "Нет описания":
                # Обрезаем описание если слишком длинное
                desc = tool_description[:100] + "..." if len(tool_description) > 100 else tool_description
                logger.info(f"   Описание: {desc}")
            
            # Обновляем агента с новым списком инструментов только если нужно
            if update_agent:
                self._update_agent()
            
            # Логируем обновленный список инструментов только если нужно
            if log_after:
                self._log_tools()
        else:
            tool_name = getattr(tool, "name", str(tool))
            logger.warning(f"⚠️ Инструмент '{tool_name}' уже зарегистрирован")

    def add_tools(self, *tools: Any):
        """
        Добавляет несколько инструментов для ChatOpenAI

        Args:
            *tools: Произвольное количество инструментов LangChain или список(ы) инструментов
        
        Examples:
            # Отдельные инструменты
            client.add_tools(tool1, tool2, tool3)
            
            # Список инструментов
            client.add_tools([tool1, tool2, tool3])
            
            # Комбинация
            client.add_tools([tool1, tool2], tool3)
        """
        if not tools:
            logger.warning("⚠️ add_tools вызван без аргументов")
            return
        
        # Распаковываем списки инструментов
        unpacked_tools = []
        for tool in tools:
            if isinstance(tool, (list, tuple)):
                # Если это список или кортеж, добавляем все элементы
                unpacked_tools.extend(tool)
            else:
                # Если это отдельный инструмент, добавляем его
                unpacked_tools.append(tool)
        
        if not unpacked_tools:
            logger.warning("⚠️ Не найдено инструментов для добавления")
            return
        
        logger.info(f"🔧 Добавление {len(unpacked_tools)} инструментов в ChatOpenAI...")
        # Добавляем инструменты без обновления агента и логирования после каждого
        for tool in unpacked_tools:
            self.add_tool(tool, update_agent=False, log_after=False)
        
        # Обновляем агента один раз после добавления всех инструментов
        self._update_agent()
        
        # Логируем финальный список инструментов один раз
        logger.info(f"✅ Все инструменты добавлены. Всего подключено: {len(self._tools)}")
        self._log_tools()

    def get_tools(self) -> List[Any]:
        """Возвращает список зарегистрированных инструментов"""
        return self._tools.copy()

    def get_tools_description_for_prompt(self) -> str:
        """
        Возвращает описание всех инструментов в формате для добавления в промпт
        
        Returns:
            Текст с описанием всех доступных инструментов
        """
        if not self._tools:
            return ""
        
        descriptions = []
        descriptions.append("\n### ДОСТУПНЫЕ ИНСТРУМЕНТЫ ###\n")
        descriptions.append("У тебя есть доступ к следующим инструментам. Используй их когда это необходимо:\n")
        
        for i, tool in enumerate(self._tools, 1):
            tool_name = getattr(tool, "name", "Unknown")
            tool_description = getattr(tool, "description", "Нет описания")
            
            # Получаем параметры инструмента, если доступны
            tool_args = ""
            if hasattr(tool, "args_schema"):
                try:
                    args_schema = tool.args_schema
                    if hasattr(args_schema, "schema"):
                        schema = args_schema.schema()
                        properties = schema.get("properties", {})
                        if properties:
                            args_list = []
                            for param_name, param_info in properties.items():
                                param_desc = param_info.get("description", "")
                                param_type = param_info.get("type", "string")
                                args_list.append(f"  - {param_name} ({param_type}): {param_desc}")
                            if args_list:
                                tool_args = "\n" + "\n".join(args_list)
                except Exception as e:
                    logger.debug(f"Не удалось получить схему аргументов для {tool_name}: {e}")
            
            descriptions.append(f"{i}. **{tool_name}**")
            descriptions.append(f"   Описание: {tool_description}")
            if tool_args:
                descriptions.append(f"   Параметры:{tool_args}")
            descriptions.append("")
        
        descriptions.append("ВАЖНО: Используй инструменты когда это необходимо для ответа на вопрос пользователя.")
        descriptions.append("Например, если пользователь спрашивает о погоде - используй соответствующий инструмент.\n")
        
        return "\n".join(descriptions)

    async def get_completion(
        self,
        messages: List
    ) -> str:
        return await self.chat_model.ainvoke({"messages": messages})
        
    async def _prepare_messages(
        self, messages: List
    ) -> List:
        """
        Подготавливает сообщения для отправки в API
        Обрезает контекст если он слишком большой
        """

        # Более точная оценка токенов для русского текста
        def estimate_message_tokens(msg):
            content = msg.content if hasattr(msg, "content") else str(msg)
            # Для русского текста: примерно 2.5-3 символа на токен
            return len(content) // 2.5

        total_estimated_tokens = sum(estimate_message_tokens(msg) for msg in messages)
        max_input_tokens = self.model_limits["max_input_tokens"]

        if total_estimated_tokens <= max_input_tokens:
            return messages

        logger.info(
            f"Контекст слишком большой ({int(total_estimated_tokens)} токенов), обрезаем до {max_input_tokens}"
        )

        # Сохраняем системные сообщения
        system_messages = [msg for msg in messages if isinstance(msg, SystemMessage)]
        other_messages = [msg for msg in messages if not isinstance(msg, SystemMessage)]

        # Рассчитываем токены системных сообщений
        system_tokens = sum(estimate_message_tokens(msg.content) for msg in system_messages)
        available_tokens = max_input_tokens - system_tokens

        if available_tokens <= 0:
            logger.warning("Системные сообщения занимают весь доступный контекст")
            return system_messages

        # Берем последние сообщения, помещающиеся в доступные токены
        current_tokens = 0
        trimmed_messages = []

        for msg in reversed(other_messages):
            msg_tokens = estimate_message_tokens(msg)
            if current_tokens + msg_tokens > available_tokens:
                break
            trimmed_messages.insert(0, msg)
            current_tokens += msg_tokens

        result_messages = system_messages + trimmed_messages
        logger.info(
            f"Контекст обрезан до {len(result_messages)} сообщений (~{int(current_tokens + system_tokens)} токенов)"
        )

        return result_messages

    async def analyze_sentiment(self, text: str) -> Dict[str, Any]:
        """
        Анализирует настроение и намерения сообщения

        Args:
            text: Текст для анализа

        Returns:
            Словарь с результатами анализа
        """
        analysis_prompt = f"""
        Проанализируй следующее сообщение пользователя и определи:
        1. Настроение (позитивное/нейтральное/негативное)
        2. Уровень заинтересованности (1-10)
        3. Готовность к покупке (1-10)
        4. Основные возражения или вопросы
        5. Рекомендуемая стратегия ответа
        
        Сообщение: "{text}"
        
        Ответь в формате JSON:
        {{
            "sentiment": "positive/neutral/negative",
            "interest_level": 1-10,
            "purchase_readiness": 1-10,
            "objections": ["список возражений"],
            "key_questions": ["ключевые вопросы"],
            "response_strategy": "рекомендуемая стратегия"
        }}
        """

        try:
            # Для анализа настроения используем более низкую температуру если поддерживается
            temp = 0.3 if not self.is_gpt5 else None
            
            model = ChatOpenAI(
                model=self.model,
                api_key=self.api_key,
                max_tokens=self.max_tokens,
                temperature=temp,
                reasoning_effort="minimal" if self._is_gpt5() else None,
                max_retries=3
            ) | StrOutputParser()

            response = await model.ainvoke(
                [
                    SystemMessage(content="Ты эксперт по анализу намерений клиентов в продажах."),
                    HumanMessage(content=analysis_prompt),
                ]
            )
            
            return json.loads(response)

        except Exception as e:
            logger.error(f"Ошибка при анализе настроения: {e}")
            # Возвращаем дефолтные значения
            return {
                "sentiment": "neutral",
                "interest_level": 5,
                "purchase_readiness": 5,
                "objections": [],
                "key_questions": [],
                "response_strategy": "continue_conversation",
            }

    async def generate_follow_up(
        self, conversation_history: List, analysis: Dict[str, Any]
    ) -> str:
        """
        Генерирует персонализированное продолжение разговора

        Args:
            conversation_history: История разговора
            analysis: Результат анализа последнего сообщения

        Returns:
            Персонализированный ответ
        """
        strategy_prompt = f"""
        На основе анализа сообщения клиента:
        - Настроение: {analysis['sentiment']}
        - Уровень заинтересованности: {analysis['interest_level']}/10
        - Готовность к покупке: {analysis['purchase_readiness']}/10
        - Возражения: {analysis['objections']}
        - Стратегия: {analysis['response_strategy']}
        
        Сгенерируй персонализированный ответ, который:
        1. Учитывает текущее настроение клиента
        2. Отвечает на его ключевые вопросы и возражения
        3. Мягко направляет к следующему этапу воронки продаж
        4. Сохраняет доверительный тон общения
        """
        
        temp = 0.8 if not self.is_gpt5 else None
        
        model = ChatOpenAI(
                model=self.model,
                api_key=self.api_key,
                max_tokens=self.max_tokens,
                temperature=temp,
                reasoning_effort="minimal" if self._is_gpt5() else None,
                max_retries=3
            ) | StrOutputParser()

        return await model.ainvoke(conversation_history + [SystemMessage(strategy_prompt)])

    async def check_api_health(self) -> bool:
        """Проверяет доступность OpenAI API"""
        try:
            test_messages = [HumanMessage(content="Привет")]
            
            model = ChatOpenAI(
                model=self.model,
                api_key=self.api_key,
                max_tokens=10,
                reasoning_effort="minimal" if self._is_gpt5() else None,
                max_retries=3
            ) | StrOutputParser()
            
            await model.ainvoke(test_messages)
            return True
        except Exception as e:
            logger.error(f"OpenAI API недоступен: {e}")
            return False

    def estimate_tokens(self, text: str) -> int:
        """Более точная оценка количества токенов в тексте"""
        # Для русского текста: примерно 2.5-3 символа на токен
        return int(len(text) / 2.5)

    async def get_available_models(self) -> List[str]:
        """Получает список доступных моделей"""
        # LangChain не предоставляет прямой доступ к списку моделей
        # Возвращаем стандартный список моделей OpenAI
        return [
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-16k",
            "gpt-4",
            "gpt-4-32k",
            "gpt-4-turbo",
            "gpt-4o",
            "gpt-4o-mini",
            "gpt-5-mini",
            "gpt-5",
        ]

    async def transcribe_audio(self, audio_file_path: str) -> str:
        """
        Распознает голосовое сообщение через Whisper API

        Args:
            audio_file_path: Путь к аудио файлу

        Returns:
            Распознанный текст
        """
        try:
            logger.info(f"🎤 Отправка аудио на распознавание: {audio_file_path}")

            with open(audio_file_path, "rb") as audio_file:
                transcript = await self.client.audio.transcriptions.create(
                    model="whisper-1", file=audio_file, language="ru"  # Русский язык
                )

            text = transcript.text
            logger.info(f"✅ Распознано {len(text)} символов: '{text[:100]}...'")
            return text

        except Exception as e:
            logger.error(f"❌ Ошибка распознавания аудио: {e}")
            return ""



"""
Функции для отправки сообщений через ИИ и от человека
"""

import logging
import time
from datetime import datetime
from typing import Any, Dict, Optional, Union

from aiogram.types import InlineKeyboardMarkup, FSInputFile
import pytz

from project_root_finder import root

PROJECT_ROOT = root

logger = logging.getLogger(__name__)


async def send_message_by_ai(
    user_id: int, message_text: str, session_id: str = None
) -> Dict[str, Any]:
    """
    Отправляет сообщение пользователю через ИИ (копирует логику process_user_message)

    Args:
        user_id: ID пользователя в Telegram
        message_text: Текст сообщения для обработки ИИ
        session_id: ID сессии чата (если не указан, будет использована активная сессия)

    Returns:
        Результат отправки
    """
    try:
        # Импортируем необходимые компоненты
        # Получаем компоненты из глобального контекста
        from ..handlers.handlers import get_global_var
        from .bot_utils import parse_ai_response, process_events

        bot = get_global_var("bot")
        supabase_client = get_global_var("supabase_client")
        openai_client = get_global_var("openai_client")
        config = get_global_var("config")
        prompt_loader = get_global_var("prompt_loader")

        # Если session_id не указан, получаем активную сессию пользователя
        if not session_id:
            session_info = await supabase_client.get_active_session(user_id)
            if not session_info:
                return {
                    "status": "error",
                    "error": "Активная сессия не найдена",
                    "user_id": user_id,
                }
            session_id = session_info["id"]

        # Загружаем системный промпт
        try:
            system_prompt = await prompt_loader.load_system_prompt()
            logger.info(f"✅ Системный промпт загружен ({len(system_prompt)} символов)")
        except Exception as e:
            logger.error(f"❌ Ошибка загрузки системного промпта: {e}")
            return {
                "status": "error",
                "error": "Не удалось загрузить системный промпт",
                "user_id": user_id,
            }

        # Сохраняем сообщение пользователя в БД
        await supabase_client.add_message(
            session_id=session_id,
            role="user",
            content=message_text,
            message_type="text",
        )
        logger.info("✅ Сообщение пользователя сохранено в БД")

        # Получаем историю сообщений
        chat_history = await supabase_client.get_chat_history(
            session_id, limit=config.MAX_CONTEXT_MESSAGES
        )
        logger.info(f"📚 Загружена история: {len(chat_history)} сообщений")

        # Добавляем текущее время
        moscow_tz = pytz.timezone("Europe/Moscow")
        current_time = datetime.now(moscow_tz)
        time_info = current_time.strftime("%H:%M, %d.%m.%Y, %A")

        # Модифицируем системный промпт, добавляя время
        system_prompt_with_time = f"""
{system_prompt}

ТЕКУЩЕЕ ВРЕМЯ: {time_info} (московское время)
"""

        # Формируем контекст для OpenAI
        messages = [{"role": "system", "content": system_prompt_with_time}]

        for msg in chat_history[-config.MAX_CONTEXT_MESSAGES :]:
            messages.append({"role": msg["role"], "content": msg["content"]})

        # Добавляем финальные инструкции
        final_instructions = await prompt_loader.load_final_instructions()
        if final_instructions:
            messages.append({"role": "system", "content": final_instructions})
            logger.info("🎯 Добавлены финальные инструкции")

        logger.info(f"📝 Контекст сформирован: {len(messages)} сообщений")

        # Отправляем действие "печатает"
        await bot.send_chat_action(user_id, "typing")

        # Получаем ответ от ИИ
        start_time = time.time()
        ai_response = await openai_client.get_completion(messages)
        processing_time = int((time.time() - start_time) * 1000)

        logger.info(f"🤖 OpenAI ответил за {processing_time}мс")

        # Обрабатываем ответ
        tokens_used = 0
        ai_metadata = {}
        response_text = ""

        if not ai_response or not ai_response.strip():
            logger.warning("❌ OpenAI вернул пустой ответ!")
            fallback_message = "Извините, произошла техническая ошибка. Попробуйте переформулировать вопрос."
            ai_response = fallback_message
            response_text = fallback_message
        else:
            tokens_used = openai_client.estimate_tokens(ai_response)
            response_text, ai_metadata = parse_ai_response(ai_response)

            if not ai_metadata:
                response_text = ai_response
                ai_metadata = {}
            elif not response_text.strip():
                response_text = ai_response

        # Обновляем этап сессии и качество лида
        if ai_metadata:
            stage = ai_metadata.get("этап")
            quality = ai_metadata.get("качество")

            if stage or quality is not None:
                await supabase_client.update_session_stage(session_id, stage, quality)
                logger.info("✅ Этап и качество обновлены в БД")

            # Обрабатываем события
            events = ai_metadata.get("събития", [])
            if events:
                logger.info(f"🔔 Обрабатываем {len(events)} событий")
                should_send_response = await process_events(session_id, events, user_id)

        # Сохраняем ответ ассистента
        await supabase_client.add_message(
            session_id=session_id,
            role="assistant",
            content=response_text,
            message_type="text",
            tokens_used=tokens_used,
            processing_time_ms=processing_time,
            ai_metadata=ai_metadata,
        )

        # Определяем финальный ответ
        if config.DEBUG_MODE:
            final_response = ai_response
        else:
            final_response = response_text

        # Проверяем, нужно ли отправлять сообщение от ИИ
        if "should_send_response" in locals() and not should_send_response:
            logger.info(
                "🔇 События запретили отправку сообщения от ИИ (message_sender), пропускаем отправку"
            )
            return {
                "status": "skipped",
                "reason": "send_ai_response=False",
                "user_id": user_id,
            }

        # Отправляем ответ пользователю напрямую через бота
        await bot.send_message(chat_id=user_id, text=final_response)

        return {
            "status": "success",
            "user_id": user_id,
            "response_text": response_text,
            "tokens_used": tokens_used,
            "processing_time_ms": processing_time,
            "events_processed": len(events) if events else 0,
        }

    except Exception as e:
        logger.error(f"❌ Ошибка в send_message_by_ai: {e}")
        return {"status": "error", "error": str(e), "user_id": user_id}


async def send_message_by_human(
    user_id: int, message_text: str, session_id: Optional[str] = None, parse_mode: str = "Markdown", reply_markup: Optional[InlineKeyboardMarkup] = None, photo: Optional[str] = None
) -> Dict[str, Any]:
    """
    Отправляет сообщение пользователю от имени человека (готовый текст или фото с подписью).

    Args:
        user_id: ID пользователя в Telegram
        message_text: Готовый текст сообщения или подпись к фото
        session_id: ID сессии (опционально, для сохранения в БД)
        parse_mode: Тип форматирования текста
        reply_markup: Клавиатура/markup (опционально)
        photo: (str) путь к локальному файлу относительно корня проекта
    Returns:
        Результат отправки
    """
    try:
        # Импортируем необходимые компоненты
        from ..handlers.handlers import get_global_var

        bot = get_global_var("bot")
        supabase_client = get_global_var("supabase_client")

        msg_type = "text"
        message = None

        if photo:
            from pathlib import Path
            photo_path = PROJECT_ROOT / photo
            if not photo_path.exists():
                raise FileNotFoundError(f"Файл с фото не найден: {photo}")
            message = await bot.send_photo(
                chat_id=user_id,
                photo=FSInputFile(str(photo_path)),
                caption=message_text,
                parse_mode=parse_mode,
                reply_markup=reply_markup
            )
            msg_type = "photo"
        else:
            message = await bot.send_message(
                chat_id=user_id,
                text=message_text,
                parse_mode=parse_mode,
                reply_markup=reply_markup,
            )

        # Если указана сессия, сохраняем сообщение в БД
        if session_id:
            await supabase_client.add_message(
                session_id=session_id,
                role="assistant",
                content=message_text,
                message_type=msg_type,
                metadata={"sent_by_human": True, "has_photo": bool(photo)},
            )
            logger.info(f"💾 Сообщение от человека сохранено в БД (photo={bool(photo)})")

        return {
            "status": "success",
            "user_id": user_id,
            "message_id": message.message_id,
            "message_text": message_text,
            "saved_to_db": bool(session_id),
            "has_photo": bool(photo)
        }

    except Exception as e:
        logger.error(f"❌ Ошибка в send_message_by_human: {e}")
        return {"status": "error", "error": str(e), "user_id": user_id}


async def send_message_to_users_by_stage(
    stage: str, message_text: str, bot_id: str, photo: Optional[str] = None
) -> Dict[str, Any]:
    """
    Отправляет сообщение (или фото с подписью) всем пользователям, находящимся на определенной стадии

    Args:
        stage: Стадия диалога (например, 'introduction', 'qualification', 'closing')
        message_text: Текст сообщения для отправки / подпись к фото
        bot_id: ID бота (если не указан, используется текущий бот)
        photo: путь к файлу с фото (относительно корня проекта, опционально)
    Returns:
        Результат отправки с количеством отправленных сообщений
    """
    try:
        from ..handlers.handlers import get_global_var
        from pathlib import Path

        bot = get_global_var("bot")
        supabase_client = get_global_var("supabase_client")
        current_bot_id = (
            get_global_var("config").BOT_ID if get_global_var("config") else bot_id
        )
        if not current_bot_id:
            return {"status": "error", "error": "Не удалось определить bot_id"}
        logger.info(
            f"🔍 Ищем пользователей на стадии '{stage}' для бота '{current_bot_id}'"
        )
        sessions_query = (
            supabase_client.client.table("sales_chat_sessions")
            .select("user_id, id, current_stage, created_at")
            .eq("status", "active")
            .eq("current_stage", stage)
        )
        if current_bot_id:
            sessions_query = sessions_query.eq("bot_id", current_bot_id)
        sessions_query = sessions_query.order("created_at", desc=True)
        sessions_data = sessions_query.execute()
        if not sessions_data.data:
            logger.info(f"📭 Пользователи на стадии '{stage}' не найдены")
            return {
                "status": "success",
                "stage": stage,
                "users_found": 0,
                "messages_sent": 0,
                "errors": [],
            }
        unique_users = {}
        for session in sessions_data.data:
            user_id = session["user_id"]
            if user_id not in unique_users:
                unique_users[user_id] = {
                    "session_id": session["id"],
                    "current_stage": session["current_stage"],
                }
        logger.info(
            f"👥 Найдено {len(unique_users)} уникальных пользователей на стадии '{stage}'"
        )
        messages_sent = 0
        errors = []
        photo_path = None
        if photo:
            photo_path = PROJECT_ROOT / photo
            if not photo_path.exists():
                raise FileNotFoundError(f"Файл с фото не найден: {photo}")
        for user_id, user_data in unique_users.items():
            session_id = user_data["session_id"]
            try:
                if photo_path:
                    msg = await bot.send_photo(chat_id=user_id, photo=FSInputFile(str(photo_path)), caption=message_text)
                    msg_type = "photo"
                else:
                    msg = await bot.send_message(chat_id=user_id, text=message_text)
                    msg_type = "text"
                await supabase_client.add_message(
                    session_id=session_id,
                    role="assistant",
                    content=message_text,
                    message_type=msg_type,
                    metadata={"sent_by_stage_broadcast": True, "target_stage": stage, "broadcast_timestamp": datetime.now().isoformat(), "has_photo": bool(photo)},
                )
                messages_sent += 1
                logger.info(
                    f"✅ Сообщение отправлено пользователю {user_id} (стадия: {stage})"
                )
            except Exception as e:
                error_msg = f"Ошибка отправки пользователю {user_id}: {str(e)}"
                errors.append(error_msg)
                logger.error(f"❌ {error_msg}")
        result = {
            "status": "success",
            "stage": stage,
            "users_found": len(unique_users),
            "messages_sent": messages_sent,
            "errors": errors,
        }
        logger.info(
            f"📊 Результат рассылки по стадии '{stage}': {messages_sent}/{len(unique_users)} сообщений отправлено"
        )
        return result
    except Exception as e:
        logger.error(f"❌ Ошибка в send_message_to_users_by_stage: {e}")
        return {"status": "error", "error": str(e), "stage": stage}


async def get_users_by_stage_stats(bot_id: Optional[str] = None) -> Dict[str, Any]:
    """
    Получает статистику пользователей по стадиям

    Args:
        bot_id: ID бота (если не указан, используется текущий бот)

    Returns:
        Статистика по стадиям с количеством пользователей
    """
    try:
        # Импортируем необходимые компоненты
        from ..handlers.handlers import get_global_var

        supabase_client = get_global_var("supabase_client")
        current_bot_id = (
            get_global_var("config").BOT_ID if get_global_var("config") else bot_id
        )

        if not current_bot_id:
            return {"status": "error", "error": "Не удалось определить bot_id"}

        logger.info(f"📊 Получаем статистику по стадиям для бота '{current_bot_id}'")

        # Получаем статистику по стадиям с user_id для подсчета уникальных пользователей
        stats_query = (
            supabase_client.client.table("sales_chat_sessions")
            .select("user_id, current_stage, created_at")
            .eq("status", "active")
        )

        # Фильтруем по bot_id если указан
        if current_bot_id:
            stats_query = stats_query.eq("bot_id", current_bot_id)

        # Сортируем по дате создания (последние сначала)
        stats_query = stats_query.order("created_at", desc=True)

        sessions_data = stats_query.execute()

        # Подсчитываем уникальных пользователей по стадиям (берем последнюю сессию каждого пользователя)
        user_stages = {}  # {user_id: stage}

        for session in sessions_data.data:
            user_id = session["user_id"]
            stage = session["current_stage"] or "unknown"

            # Если пользователь еще не добавлен, добавляем его стадию (первая встреченная - самая последняя)
            if user_id not in user_stages:
                user_stages[user_id] = stage

        # Подсчитываем количество пользователей по стадиям
        stage_stats = {}
        for stage in user_stages.values():
            stage_stats[stage] = stage_stats.get(stage, 0) + 1

        total_users = len(user_stages)

        # Сортируем по количеству пользователей (по убыванию)
        sorted_stages = sorted(stage_stats.items(), key=lambda x: x[1], reverse=True)

        result = {
            "status": "success",
            "bot_id": current_bot_id,
            "total_active_users": total_users,
            "stages": dict(sorted_stages),
            "stages_list": sorted_stages,
        }

        logger.info(f"📊 Статистика по стадиям: {total_users} активных пользователей")
        for stage, count in sorted_stages:
            logger.info(f"   {stage}: {count} пользователей")

        return result

    except Exception as e:
        logger.error(f"❌ Ошибка в get_users_by_stage_stats: {e}")
        return {"status": "error", "error": str(e), "bot_id": bot_id}


async def send_message(
    message,
    text: str,
    supabase_client,
    files_list: list = [],
    directories_list: list = [],
    parse_mode: str = "Markdown",
    **kwargs,
):
    """
    Пользовательская функция для отправки сообщений с файлами и кнопками

    Args:
        message: Message объект от aiogram
        text: Текст сообщения
        supabase_client: SupabaseClient для работы с БД
        files_list: Список файлов для отправки
        directories_list: Список каталогов (отправятся все файлы)
        parse_mode: Режим парсинга ('Markdown', 'HTML' или None)
        **kwargs: Дополнительные параметры (reply_markup и т.д.)

    Returns:
        Message объект отправленного сообщения или None

    Example:
        from smart_bot_factory.message import send_message
        from aiogram.types import InlineKeyboardMarkup, InlineKeyboardButton

        keyboard = InlineKeyboardMarkup(inline_keyboard=[
            [InlineKeyboardButton(text="Кнопка", callback_data="action")]
        ])

        await send_message(
            message=message,
            text="Привет!",
            supabase_client=supabase_client,
            files_list=["file.pdf"],
            parse_mode="Markdown",
            reply_markup=keyboard
        )
    """
    from pathlib import Path

    from aiogram.types import FSInputFile
    from aiogram.utils.media_group import MediaGroupBuilder

    logger.info("📤 send_message вызвана:")
    logger.info(f"   👤 Пользователь: {message.from_user.id}")
    logger.info(f"   📝 Длина текста: {len(text)} символов")
    logger.info(f"   🔧 Parse mode: {parse_mode}")

    try:
        user_id = message.from_user.id

        # Устанавливаем parse_mode (None если передана строка 'None')
        actual_parse_mode = None if parse_mode == "None" else parse_mode

        # Текст уже готов, используем как есть
        final_text = text

        # Работаем с переданными файлами и каталогами
        logger.info(f"   📦 Передано файлов: {files_list}")
        logger.info(f"   📂 Передано каталогов: {directories_list}")

        # Получаем список уже отправленных файлов и каталогов
        sent_files = await supabase_client.get_sent_files(user_id)
        sent_directories = await supabase_client.get_sent_directories(user_id)

        logger.info(f"   📋 Уже отправлено файлов: {sent_files}")
        logger.info(f"   📋 Уже отправлено каталогов: {sent_directories}")

        # Фильтруем файлы и каталоги, которые уже отправлялись
        actual_files_list = [f for f in files_list if f not in sent_files]
        actual_directories_list = [
            d for d in directories_list if str(d) not in sent_directories
        ]

        logger.info(f"   🆕 После фильтрации файлов: {actual_files_list}")
        logger.info(f"   🆕 После фильтрации каталогов: {actual_directories_list}")

        # Проверяем, что есть что отправлять
        if not final_text or not final_text.strip():
            logger.error("❌ КРИТИЧЕСКАЯ ОШИБКА: final_text пуст после обработки!")
            logger.error(f"   Исходный text: '{text[:200]}...'")
            final_text = "Ошибка формирования ответа. Попробуйте еще раз."

        logger.info(f"📱 Подготовка сообщения: {len(final_text)} символов")
        logger.info(f"   📦 Файлов для обработки: {actual_files_list}")
        logger.info(f"   📂 Каталогов для обработки: {actual_directories_list}")

        # Проверяем наличие файлов для отправки
        if actual_files_list or actual_directories_list:
            # Функция определения типа медиа по расширению
            def get_media_type(file_path: str) -> str:
                ext = Path(file_path).suffix.lower()
                if ext in {".jpg", ".jpeg", ".png"}:
                    return "photo"
                elif ext in {".mp4", ".mov"}:
                    return "video"
                else:
                    return "document"

            # Создаем списки для разных типов файлов
            video_files = []
            photo_files = []
            document_files = []

            # Функция обработки файла
            def process_file(file_path: Path, source: str = ""):
                if file_path.is_file():
                    media_type = get_media_type(str(file_path))
                    if media_type == "video":
                        video_files.append(file_path)
                        logger.info(
                            f"   🎥 Добавлено видео{f' из {source}' if source else ''}: {file_path.name}"
                        )
                    elif media_type == "photo":
                        photo_files.append(file_path)
                        logger.info(
                            f"   📸 Добавлено фото{f' из {source}' if source else ''}: {file_path.name}"
                        )
                    else:
                        document_files.append(file_path)
                        logger.info(
                            f"   📄 Добавлен документ{f' из {source}' if source else ''}: {file_path.name}"
                        )
                else:
                    logger.warning(f"   ⚠️ Файл не найден: {file_path}")

            # Обрабатываем прямые файлы
            for file_name in actual_files_list:
                try:
                    process_file(Path(f"files/{file_name}"))
                except Exception as e:
                    logger.error(f"   ❌ Ошибка обработки файла {file_name}: {e}")

            # Обрабатываем файлы из каталогов
            for dir_name in actual_directories_list:
                dir_name = Path(dir_name)
                try:
                    if dir_name.is_dir():
                        for file_path in dir_name.iterdir():
                            try:
                                process_file(file_path, dir_name)
                            except Exception as e:
                                logger.error(
                                    f"   ❌ Ошибка обработки файла {file_path}: {e}"
                                )
                    else:
                        logger.warning(f"   ⚠️ Каталог не найден: {dir_name}")
                except Exception as e:
                    logger.error(f"   ❌ Ошибка обработки каталога {dir_name}: {e}")

            # Списки для отслеживания реально отправленных файлов
            sent_files_to_save = []
            sent_dirs_to_save = []

            # 1. Отправляем видео (если есть)
            if video_files:
                video_group = MediaGroupBuilder()
                for file_path in video_files:
                    video_group.add_video(media=FSInputFile(str(file_path)))

                videos = video_group.build()
                if videos:
                    await message.answer_media_group(media=videos)
                    logger.info(f"   ✅ Отправлено {len(videos)} видео")

            # 2. Отправляем фото (если есть)
            if photo_files:
                photo_group = MediaGroupBuilder()
                for file_path in photo_files:
                    photo_group.add_photo(media=FSInputFile(str(file_path)))

                photos = photo_group.build()
                if photos:
                    await message.answer_media_group(media=photos)
                    logger.info(f"   ✅ Отправлено {len(photos)} фото")

            # 3. Отправляем текст
            result = await message.answer(
                final_text, parse_mode=actual_parse_mode, **kwargs
            )
            logger.info("   ✅ Отправлен текст сообщения")

            # 4. Отправляем документы (если есть)
            if document_files:
                doc_group = MediaGroupBuilder()
                for file_path in document_files:
                    doc_group.add_document(media=FSInputFile(str(file_path)))

                docs = doc_group.build()
                if docs:
                    await message.answer_media_group(media=docs)
                    logger.info(f"   ✅ Отправлено {len(docs)} документов")

            # 5. Собираем список реально отправленных файлов и каталогов
            if video_files or photo_files or document_files:
                sent_files_to_save.extend(actual_files_list)
                logger.info(
                    f"   📝 Добавляем в список для сохранения файлы: {actual_files_list}"
                )
                sent_dirs_to_save.extend([str(d) for d in actual_directories_list])
                logger.info(
                    f"   📝 Добавляем в список для сохранения каталоги: {actual_directories_list}"
                )

            # 6. Обновляем информацию в БД
            if sent_files_to_save or sent_dirs_to_save:
                try:
                    if sent_files_to_save:
                        logger.info(f"   💾 Сохраняем файлы в БД: {sent_files_to_save}")
                        await supabase_client.add_sent_files(
                            user_id, sent_files_to_save
                        )
                    if sent_dirs_to_save:
                        logger.info(
                            f"   💾 Сохраняем каталоги в БД: {sent_dirs_to_save}"
                        )
                        await supabase_client.add_sent_directories(
                            user_id, sent_dirs_to_save
                        )
                    logger.info(
                        "   ✅ Обновлена информация о отправленных файлах в БД"
                    )
                except Exception as e:
                    logger.error(
                        f"   ❌ Ошибка обновления информации о файлах в БД: {e}"
                    )
            else:
                logger.info("   ℹ️ Нет новых файлов для сохранения в БД")

            return result
        else:
            # Если нет файлов, отправляем просто текст
            logger.info("   ⚠️ Нет файлов для отправки, отправляем как текст")
            result = await message.answer(
                final_text, parse_mode=actual_parse_mode, **kwargs
            )
            return result

    except Exception as e:
        # Проверяем, является ли ошибка блокировкой бота
        if "Forbidden: bot was blocked by the user" in str(e):
            logger.warning(f"🚫 Бот заблокирован пользователем {user_id}")
            return None
        elif "TelegramForbiddenError" in str(type(e).__name__):
            logger.warning(f"🚫 Бот заблокирован пользователем {user_id}")
            return None

        logger.error(f"❌ ОШИБКА в send_message: {e}")
        logger.exception("Полный стек ошибки send_message:")

        # Пытаемся отправить простое сообщение без форматирования
        try:
            fallback_text = "Произошла ошибка при отправке ответа. Попробуйте еще раз."
            result = await message.answer(fallback_text)
            logger.info("✅ Запасное сообщение отправлено")
            return result
        except Exception as e2:
            if "Forbidden: bot was blocked by the user" in str(e2):
                logger.warning(
                    f"🚫 Бот заблокирован пользователем {user_id} (fallback)"
                )
                return None
            elif "TelegramForbiddenError" in str(type(e2).__name__):
                logger.warning(
                    f"🚫 Бот заблокирован пользователем {user_id} (fallback)"
                )
                return None

            logger.error(f"❌ Даже запасное сообщение не отправилось: {e2}")
            raise



# Исправленный admin_logic.py с правильной обработкой диалогов

import logging

from aiogram import F, Router
from aiogram.filters import Command, StateFilter
from aiogram.fsm.context import FSMContext
from aiogram.types import (CallbackQuery, InlineKeyboardButton,
                           InlineKeyboardMarkup, Message)

# Импортируем состояния
from ..core.states import AdminStates

logger = logging.getLogger(__name__)

# Создаем роутер для админских обработчиков
admin_router = Router()


def setup_admin_handlers(dp):
    """Настройка админских обработчиков"""
    dp.include_router(admin_router)


@admin_router.message(Command(commands=["отмена", "cancel"]))
async def cancel_handler(message: Message, state: FSMContext):
    """Отмена текущего действия и очистка state"""
    from ..handlers.handlers import get_global_var

    admin_manager = get_global_var("admin_manager")

    # Получаем текущий state
    current_state = await state.get_state()

    # Очищаем временные файлы если это создание события
    if current_state and current_state.startswith("AdminStates:create_event"):
        from .admin_events import cleanup_temp_files

        await cleanup_temp_files(state)

    # Очищаем state
    await state.clear()

    if current_state:
        logger.info(
            f"State очищен для пользователя {message.from_user.id}: {current_state}"
        )

        # Если это админ, возвращаем в админ режим
        if admin_manager.is_admin(message.from_user.id):
            await state.set_state(AdminStates.admin_mode)
            await message.answer(
                "✅ Текущее действие отменено\n"
                "Вы вернулись в админ режим\n\n"
                "Используйте /admin для просмотра доступных команд",
                parse_mode="Markdown",
            )
        else:
            await message.answer(
                "✅ Текущее действие отменено\n\n"
                "Используйте /start для начала работы",
                parse_mode="Markdown",
            )
    else:
        await message.answer(
            "ℹ️ Нет активных действий для отмены", parse_mode="Markdown"
        )


async def admin_start_handler(message: Message, state: FSMContext):
    """Обработчик /start для админов в режиме администратора"""
    from ..handlers.handlers import get_global_var

    admin_manager = get_global_var("admin_manager")

    await state.set_state(AdminStates.admin_mode)

    admin_status = admin_manager.get_admin_mode_text(message.from_user.id)

    # Основное меню админа
    keyboard = InlineKeyboardMarkup(
        inline_keyboard=[
            [InlineKeyboardButton(text="📊 Статистика", callback_data="admin_stats")],
            [
                InlineKeyboardButton(
                    text="💬 Активные чаты", callback_data="admin_active_chats"
                )
            ],
            [
                InlineKeyboardButton(
                    text="🔄 Режим польз.", callback_data="admin_toggle_mode"
                )
            ],
        ]
    )

    welcome_text = f"""
{admin_status}

🎛️ **Панель администратора**

Доступные команды:
• `/стат` - статистика воронки
• `/история user_id` - история пользователя
• `/чат user_id` - начать диалог
• `/чаты` - активные диалоги
• `/стоп` - завершить диалог
• `/админ` - переключить режим
• `/отмена` - отменить текущее действие

📅 **Управление событиями:**
• `/создать_событие` - создать новое событие
• `/список_событий` - список активных событий
• `/удалить_событие название` - отменить событие
"""

    await message.answer(welcome_text, reply_markup=keyboard, parse_mode="Markdown")


@admin_router.message(Command(commands=["стат", "stats"]))
async def admin_stats_handler(message: Message, state: FSMContext):
    """Статистика воронки"""
    from ..handlers.handlers import get_global_var

    admin_manager = get_global_var("admin_manager")
    analytics_manager = get_global_var("analytics_manager")

    if not admin_manager.is_admin(message.from_user.id):
        return

    try:
        # Получаем статистику
        funnel_stats = await analytics_manager.get_funnel_stats(7)
        events_stats = await analytics_manager.get_events_stats(7)

        # Форматируем ответ
        funnel_text = analytics_manager.format_funnel_stats(funnel_stats)
        events_text = analytics_manager.format_events_stats(events_stats)

        full_text = f"{funnel_text}\n\n{events_text}"

        await message.answer(full_text)

    except Exception as e:
        logger.error(f"Ошибка получения статистики: {e}")
        await message.answer("❌ Ошибка получения статистики")


@admin_router.message(Command(commands=["история", "history"]))
async def admin_history_handler(message: Message, state: FSMContext):
    """История пользователя"""
    from ..handlers.handlers import get_global_var

    admin_manager = get_global_var("admin_manager")
    analytics_manager = get_global_var("analytics_manager")

    if not admin_manager.is_admin(message.from_user.id):
        return

    try:
        parts = message.text.split()
        if len(parts) < 2:
            await message.answer("Укажите ID пользователя: /история 123456789")
            return

        user_id = int(parts[1])

        # Получаем историю (та же функция что использует кнопка)
        journey = await analytics_manager.get_user_journey(user_id)

        if not journey:
            await message.answer(f"❌ У пользователя {user_id} нет активной сессии")
            return

        # Используем ту же функцию форматирования что и кнопка
        history_text = analytics_manager.format_user_journey(user_id, journey)

        await message.answer(history_text)

    except ValueError:
        await message.answer("❌ Неверный формат ID пользователя")
    except Exception as e:
        logger.error(f"Ошибка получения истории: {e}")
        await message.answer("❌ Ошибка получения истории")


@admin_router.message(Command(commands=["чат", "chat"]))
async def admin_chat_handler(message: Message, state: FSMContext):
    """Начать диалог с пользователем"""
    from ..handlers.handlers import get_global_var

    admin_manager = get_global_var("admin_manager")
    supabase_client = get_global_var("supabase_client")
    conversation_manager = get_global_var("conversation_manager")

    if not admin_manager.is_admin(message.from_user.id):
        return

    try:
        # Парсим user_id из команды
        parts = message.text.split()
        if len(parts) < 2:
            await message.answer("Укажите ID пользователя: /чат 123456789")
            return

        user_id = int(parts[1])
        admin_id = message.from_user.id

        logger.info(
            f"👑 Админ {admin_id} хочет начать диалог с пользователем {user_id}"
        )

        # Проверяем, есть ли активная сессия у пользователя
        session_info = await supabase_client.get_active_session(user_id)
        if not session_info:
            await message.answer(f"❌ У пользователя {user_id} нет активной сессии")
            logger.warning(f"❌ У пользователя {user_id} нет активной сессии")
            return

        logger.info(
            f"✅ У пользователя {user_id} есть активная сессия: {session_info['id']}"
        )

        # Начинаем диалог
        logger.info("🚀 Запускаем создание диалога...")
        success = await conversation_manager.start_admin_conversation(admin_id, user_id)

        if success:
            # ✅ ИСПРАВЛЕНИЕ: Правильно переключаем состояние админа
            await state.set_state(AdminStates.in_conversation)
            await state.update_data(conversation_user_id=user_id)

            await message.answer(
                f"✅ Диалог с пользователем {user_id} начат\n💬 Ваши сообщения будут переданы пользователю\n⏹️ Используйте /стоп для завершения"
            )
            logger.info(
                "✅ Диалог успешно создан, админ переключен в состояние in_conversation"
            )
        else:
            await message.answer(
                f"❌ Не удалось начать диалог с пользователем {user_id}"
            )
            logger.error("❌ Не удалось создать диалог")

    except ValueError:
        await message.answer("❌ Неверный формат ID пользователя")
        logger.error(f"❌ Неверный формат ID пользователя: {message.text}")
    except Exception as e:
        logger.error(f"❌ Ошибка начала диалога: {e}")
        await message.answer("❌ Ошибка начала диалога")


@admin_router.message(Command(commands=["чаты", "chats"]))
async def admin_active_chats_command(message: Message, state: FSMContext):
    """Показать активные диалоги админов"""
    from ..handlers.handlers import get_global_var

    admin_manager = get_global_var("admin_manager")
    conversation_manager = get_global_var("conversation_manager")

    if not admin_manager.is_admin(message.from_user.id):
        return

    try:
        conversations = await conversation_manager.get_active_conversations()
        formatted_text = conversation_manager.format_active_conversations(conversations)

        # ✅ ИСПРАВЛЕНИЕ: Убираем parse_mode='Markdown' чтобы избежать ошибок парсинга
        await message.answer(formatted_text)

    except Exception as e:
        logger.error(f"Ошибка получения активных чатов: {e}")
        await message.answer("❌ Ошибка получения активных диалогов")


@admin_router.message(Command(commands=["стоп", "stop"]))
async def admin_stop_handler(message: Message, state: FSMContext):
    """Завершить диалог"""
    from ..handlers.handlers import get_global_var

    admin_manager = get_global_var("admin_manager")
    conversation_manager = get_global_var("conversation_manager")

    if not admin_manager.is_admin(message.from_user.id):
        return

    try:
        admin_id = message.from_user.id

        # Проверяем есть ли активный диалог
        conversation = await conversation_manager.get_admin_active_conversation(
            admin_id
        )

        if conversation:
            user_id = conversation["user_id"]
            logger.info(
                f"🛑 Завершаем диалог админа {admin_id} с пользователем {user_id}"
            )

            success = await conversation_manager.end_admin_conversation(admin_id)

            if success:
                # ✅ ИСПРАВЛЕНИЕ: Правильно переключаем состояние обратно
                await state.set_state(AdminStates.admin_mode)
                await state.update_data(conversation_user_id=None)

                await message.answer(f"✅ Диалог с пользователем {user_id} завершен")
                logger.info("✅ Диалог завершен, админ переключен в admin_mode")
            else:
                await message.answer("❌ Ошибка завершения диалога")
        else:
            await message.answer("❌ Нет активного диалога")
            logger.info(f"❌ У админа {admin_id} нет активного диалога")

    except Exception as e:
        logger.error(f"Ошибка завершения диалога: {e}")
        await message.answer("❌ Ошибка завершения диалога")


@admin_router.message(Command(commands=["админ", "admin"]))
async def admin_toggle_handler(message: Message, state: FSMContext):
    """Переключение режима админа"""
    from ..handlers.handlers import get_global_var

    admin_manager = get_global_var("admin_manager")

    if not admin_manager.is_admin(message.from_user.id):
        return

    new_mode = admin_manager.toggle_admin_mode(message.from_user.id)

    if new_mode:
        # Переключились в режим админа
        await admin_start_handler(message, state)
    else:
        # Переключились в режим пользователя
        await state.clear()
        await message.answer(
            "🔄 Переключен в режим пользователя\nНапишите /start для начала диалога"
        )


@admin_router.message(Command("debug_chat"))
async def debug_chat_handler(message: Message, state: FSMContext):
    """Отладка диалогов админов"""
    from ..handlers.handlers import get_global_var

    admin_manager = get_global_var("admin_manager")
    conversation_manager = get_global_var("conversation_manager")
    supabase_client = get_global_var("supabase_client")

    if not admin_manager.is_admin(message.from_user.id):
        return

    parts = message.text.split()
    if len(parts) < 2:
        await message.answer("Использование: /debug_chat USER_ID")
        return

    try:
        user_id = int(parts[1])

        # 1. Проверяем запись в БД
        conversation = await conversation_manager.is_user_in_admin_chat(user_id)

        debug_info = [
            f"🔍 ОТЛАДКА ДИАЛОГА С {user_id}",
            "",
            f"📊 Диалог в БД: {'✅' if conversation else '❌'}",
        ]

        if conversation:
            debug_info.extend(
                [
                    f"👑 Админ: {conversation['admin_id']}",
                    f"🕐 Начат: {conversation['started_at']}",
                ]
            )

        # 2. Проверяем активную сессию пользователя
        session_info = await supabase_client.get_active_session(user_id)
        debug_info.append(f"🎯 Активная сессия: {'✅' if session_info else '❌'}")

        if session_info:
            debug_info.append(f"📝 ID сессии: {session_info['id']}")

        # 3. Проверяем состояние пользователя (если он онлайн)
        debug_info.append("")
        debug_info.append(
            "ℹ️ Для проверки состояния пользователь должен написать что-то"
        )

        await message.answer("\n".join(debug_info))

    except Exception as e:
        await message.answer(f"❌ Ошибка: {e}")
        logger.error(f"Ошибка отладки: {e}")


@admin_router.callback_query(F.data.startswith("admin_"))
async def admin_callback_handler(callback: CallbackQuery, state: FSMContext):
    """Обработчик callback кнопок админов"""
    from ..handlers.handlers import get_global_var

    admin_manager = get_global_var("admin_manager")
    analytics_manager = get_global_var("analytics_manager")
    conversation_manager = get_global_var("conversation_manager")

    if not admin_manager.is_admin(callback.from_user.id):
        await callback.answer("Нет доступа")
        return

    data = callback.data

    try:
        if data == "admin_stats":
            # Показываем статистику
            funnel_stats = await analytics_manager.get_funnel_stats(7)
            events_stats = await analytics_manager.get_events_stats(7)

            funnel_text = analytics_manager.format_funnel_stats(funnel_stats)
            events_text = analytics_manager.format_events_stats(events_stats)

            await callback.message.answer(f"{funnel_text}\n\n{events_text}")

        elif data == "admin_toggle_mode":
            # Переключаем режим
            new_mode = admin_manager.toggle_admin_mode(callback.from_user.id)
            mode_text = "администратор" if new_mode else "пользователь"
            await callback.answer(f"Режим переключен: {mode_text}")

            if not new_mode:
                await state.clear()
                await callback.message.answer("🔄 Теперь вы в режиме пользователя")

        elif data == "admin_active_chats":
            # Показываем активные диалоги
            conversations = await conversation_manager.get_active_conversations()
            formatted_text = conversation_manager.format_active_conversations(
                conversations
            )

            # ✅ ИСПРАВЛЕНИЕ: Убираем parse_mode='Markdown'
            await callback.message.answer(formatted_text)

        elif data.startswith("admin_history_"):
            user_id = int(data.split("_")[2])
            journey = await analytics_manager.get_user_journey(user_id)
            history_text = analytics_manager.format_user_journey(user_id, journey)
            await callback.message.answer(history_text)

        elif data.startswith("admin_end_"):
            user_id = int(data.split("_")[2])

            # Проверяем есть ли активный диалог
            conversation = await conversation_manager.get_admin_active_conversation(
                callback.from_user.id
            )

            if conversation and conversation["user_id"] == user_id:
                await conversation_manager.end_admin_conversation(callback.from_user.id)

                # ✅ ИСПРАВЛЕНИЕ: Правильно переключаем состояние
                await state.set_state(AdminStates.admin_mode)
                await state.update_data(conversation_user_id=None)

                await callback.answer("Диалог завершен")
                await callback.message.answer(
                    f"✅ Диалог с пользователем {user_id} завершен"
                )
                logger.info(
                    "✅ Диалог завершен через кнопку, админ переключен в admin_mode"
                )
            else:
                await callback.answer("Диалог не найден")

        elif data.startswith("admin_chat_"):
            user_id = int(data.split("_")[2])
            admin_id = callback.from_user.id

            success = await conversation_manager.start_admin_conversation(
                admin_id, user_id
            )
            if success:
                # ✅ ИСПРАВЛЕНИЕ: Правильно переключаем состояние
                await state.set_state(AdminStates.in_conversation)
                await state.update_data(conversation_user_id=user_id)

                await callback.answer("Диалог начат")
                await callback.message.answer(
                    f"✅ Диалог с пользователем {user_id} начат"
                )
                logger.info(
                    "✅ Диалог начат через кнопку, админ переключен в in_conversation"
                )
            else:
                await callback.answer("Не удалось начать диалог")

        await callback.answer()

    except Exception as e:
        logger.error(f"Ошибка обработки callback {data}: {e}")
        await callback.answer("Ошибка")


@admin_router.message(
    StateFilter(AdminStates.admin_mode, AdminStates.in_conversation),
    F.text,
    lambda message: not message.text.startswith("/"),
)
async def admin_message_handler(message: Message, state: FSMContext):
    """Обработчик сообщений админов"""
    from ..handlers.handlers import get_global_var

    admin_manager = get_global_var("admin_manager")
    conversation_manager = get_global_var("conversation_manager")

    if not admin_manager.is_admin(message.from_user.id):
        return

    try:
        logger.info(
            f"👑 Получено сообщение от админа {message.from_user.id}: '{message.text}'"
        )

        # Пытаемся обработать как админское сообщение
        handled = await conversation_manager.route_admin_message(message, state)

        if handled:
            logger.info("✅ Сообщение админа обработано и переслано пользователю")
        else:
            # Не админское сообщение - показываем справку
            logger.info("❌ Сообщение админа не обработано, показываем справку")
            await message.answer(
                """
👑 **Режим администратора**

Доступные команды:
• `/стат` - статистика воронки
• `/история user_id` - история пользователя  
• `/чат user_id` - начать диалог
• `/стоп` - завершить диалог
• `/админ` - переключить режим

💡 Если вы в диалоге с пользователем, просто напишите сообщение - оно будет переслано пользователю.
""",
                parse_mode="Markdown",
            )

    except Exception as e:
        logger.error(f"Ошибка обработки сообщения админа: {e}")
        await message.answer("❌ Ошибка обработки команды")
